"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def abort(
    hub,
    ctx,
    bucket: str,
    key: str,
    upload_id: str,
    request_payer: str = None,
    expected_bucket_owner: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    This action aborts a multipart upload. After a multipart upload is aborted, no additional parts can be uploaded using that upload ID. The storage consumed by any previously uploaded parts will be freed. However, if any part uploads are currently in progress, those part uploads might or might not succeed. As a result, it might be necessary to abort a given multipart upload multiple times in order to completely free all storage consumed by all parts.  To verify that all parts have been removed, so you don't get charged for the part storage, you should call the ListParts action and ensure that the parts list is empty. For information about permissions required to use the multipart upload, see Multipart Upload and Permissions. The following operations are related to AbortMultipartUpload:    CreateMultipartUpload     UploadPart     CompleteMultipartUpload     ListParts     ListMultipartUploads

    Args:
        bucket(str): The bucket name to which the upload was taking place.  When using this action with an access point, you must direct requests to the access point hostname. The access point hostname takes the form AccessPointName-AccountId.s3-accesspoint.Region.amazonaws.com. When using this action with an access point through the AWS SDKs, you provide the access point ARN in place of the bucket name. For more information about access point ARNs, see Using access points in the Amazon S3 User Guide. When using this action with Amazon S3 on Outposts, you must direct requests to the S3 on Outposts hostname. The S3 on Outposts hostname takes the form AccessPointName-AccountId.outpostID.s3-outposts.Region.amazonaws.com. When using this action using S3 on Outposts through the AWS SDKs, you provide the Outposts bucket ARN in place of the bucket name. For more information about S3 on Outposts ARNs, see Using S3 on Outposts in the Amazon S3 User Guide.
        key(str): Key of the object for which the multipart upload was initiated.
        upload_id(str): Upload ID that identifies the multipart upload.
        request_payer(str, optional): Confirms that the requester knows that they will be charged for the request. Bucket owners need not specify this parameter in their requests. For information about downloading objects from requester pays buckets, see Downloading Objects in Requestor Pays Buckets in the Amazon S3 User Guide. Defaults to None.
        expected_bucket_owner(str, optional): The account ID of the expected bucket owner. If the bucket is owned by a different account, the request will fail with an HTTP 403 (Access Denied) error. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.s3.multipart_upload.abort(
                ctx, bucket=value, key=value, upload_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.s3.multipart_upload.abort bucket=value, key=value, upload_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="s3",
        operation="abort_multipart_upload",
        op_kwargs={
            "Bucket": bucket,
            "Key": key,
            "UploadId": upload_id,
            "RequestPayer": request_payer,
            "ExpectedBucketOwner": expected_bucket_owner,
        },
    )


async def complete(
    hub,
    ctx,
    bucket: str,
    key: str,
    upload_id: str,
    multipart_upload: Dict = None,
    request_payer: str = None,
    expected_bucket_owner: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Completes a multipart upload by assembling previously uploaded parts. You first initiate the multipart upload and then upload all parts using the UploadPart operation. After successfully uploading all relevant parts of an upload, you call this action to complete the upload. Upon receiving this request, Amazon S3 concatenates all the parts in ascending order by part number to create a new object. In the Complete Multipart Upload request, you must provide the parts list. You must ensure that the parts list is complete. This action concatenates the parts that you provide in the list. For each part in the list, you must provide the part number and the ETag value, returned after that part was uploaded. Processing of a Complete Multipart Upload request could take several minutes to complete. After Amazon S3 begins processing the request, it sends an HTTP response header that specifies a 200 OK response. While processing is in progress, Amazon S3 periodically sends white space characters to keep the connection from timing out. Because a request could fail after the initial 200 OK response has been sent, it is important that you check the response body to determine whether the request succeeded. Note that if CompleteMultipartUpload fails, applications should be prepared to retry the failed requests. For more information, see Amazon S3 Error Best Practices. For more information about multipart uploads, see Uploading Objects Using Multipart Upload. For information about permissions required to use the multipart upload API, see Multipart Upload and Permissions.  CompleteMultipartUpload has the following special errors:   Error code: EntityTooSmall    Description: Your proposed upload is smaller than the minimum allowed object size. Each part must be at least 5 MB in size, except the last part.   400 Bad Request     Error code: InvalidPart    Description: One or more of the specified parts could not be found. The part might not have been uploaded, or the specified entity tag might not have matched the part's entity tag.   400 Bad Request     Error code: InvalidPartOrder    Description: The list of parts was not in ascending order. The parts list must be specified in order by part number.   400 Bad Request     Error code: NoSuchUpload    Description: The specified multipart upload does not exist. The upload ID might be invalid, or the multipart upload might have been aborted or completed.   404 Not Found     The following operations are related to CompleteMultipartUpload:    CreateMultipartUpload     UploadPart     AbortMultipartUpload     ListParts     ListMultipartUploads

    Args:
        bucket(str): Name of the bucket to which the multipart upload was initiated.
        key(str): Object key for which the multipart upload was initiated.
        multipart_upload(Dict, optional): The container for the multipart upload request information. Defaults to None.
        upload_id(str): ID for the initiated multipart upload.
        request_payer(str, optional): Confirms that the requester knows that they will be charged for the request. Bucket owners need not specify this parameter in their requests. For information about downloading objects from requester pays buckets, see Downloading Objects in Requestor Pays Buckets in the Amazon S3 User Guide. Defaults to None.
        expected_bucket_owner(str, optional): The account ID of the expected bucket owner. If the bucket is owned by a different account, the request will fail with an HTTP 403 (Access Denied) error. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.s3.multipart_upload.complete(
                ctx, bucket=value, key=value, upload_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.s3.multipart_upload.complete bucket=value, key=value, upload_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="s3",
        operation="complete_multipart_upload",
        op_kwargs={
            "Bucket": bucket,
            "Key": key,
            "MultipartUpload": multipart_upload,
            "UploadId": upload_id,
            "RequestPayer": request_payer,
            "ExpectedBucketOwner": expected_bucket_owner,
        },
    )


async def create(
    hub,
    ctx,
    bucket: str,
    key: str,
    acl: str = None,
    cache_control: str = None,
    content_disposition: str = None,
    content_encoding: str = None,
    content_language: str = None,
    content_type: str = None,
    expires: str = None,
    grant_full_control: str = None,
    grant_read: str = None,
    grant_read_acp: str = None,
    grant_write_acp: str = None,
    metadata: Dict = None,
    server_side_encryption: str = None,
    storage_class: str = None,
    website_redirect_location: str = None,
    sse_customer_algorithm: str = None,
    sse_customer_key: str = None,
    sse_customer_key_md5: str = None,
    ssekms_key_id: str = None,
    ssekms_encryption_context: str = None,
    bucket_key_enabled: bool = None,
    request_payer: str = None,
    tagging: str = None,
    object_lock_mode: str = None,
    object_lock_retain_until_date: str = None,
    object_lock_legal_hold_status: str = None,
    expected_bucket_owner: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    This action initiates a multipart upload and returns an upload ID. This upload ID is used to associate all of the parts in the specific multipart upload. You specify this upload ID in each of your subsequent upload part requests (see UploadPart). You also include this upload ID in the final request to either complete or abort the multipart upload request. For more information about multipart uploads, see Multipart Upload Overview. If you have configured a lifecycle rule to abort incomplete multipart uploads, the upload must complete within the number of days specified in the bucket lifecycle configuration. Otherwise, the incomplete multipart upload becomes eligible for an abort action and Amazon S3 aborts the multipart upload. For more information, see Aborting Incomplete Multipart Uploads Using a Bucket Lifecycle Policy. For information about the permissions required to use the multipart upload API, see Multipart Upload and Permissions. For request signing, multipart upload is just a series of regular requests. You initiate a multipart upload, send one or more requests to upload parts, and then complete the multipart upload process. You sign each request individually. There is nothing special about signing multipart upload requests. For more information about signing, see Authenticating Requests (AWS Signature Version 4).   After you initiate a multipart upload and upload one or more parts, to stop being charged for storing the uploaded parts, you must either complete or abort the multipart upload. Amazon S3 frees up the space used to store the parts and stop charging you for storing them only after you either complete or abort a multipart upload.   You can optionally request server-side encryption. For server-side encryption, Amazon S3 encrypts your data as it writes it to disks in its data centers and decrypts it when you access it. You can provide your own encryption key, or use AWS Key Management Service (AWS KMS) customer master keys (CMKs) or Amazon S3-managed encryption keys. If you choose to provide your own encryption key, the request headers you provide in UploadPart and UploadPartCopy requests must match the headers you used in the request to initiate the upload by using CreateMultipartUpload.  To perform a multipart upload with encryption using an AWS KMS CMK, the requester must have permission to the kms:Decrypt and kms:GenerateDataKey* actions on the key. These permissions are required because Amazon S3 must decrypt and read data from the encrypted file parts before it completes the multipart upload. For more information, see Multipart upload API and permissions in the Amazon S3 User Guide. If your AWS Identity and Access Management (IAM) user or role is in the same AWS account as the AWS KMS CMK, then you must have these permissions on the key policy. If your IAM user or role belongs to a different account than the key, then you must have the permissions on both the key policy and your IAM user or role.  For more information, see Protecting Data Using Server-Side Encryption.  Access Permissions  When copying an object, you can optionally specify the accounts or groups that should be granted specific permissions on the new object. There are two ways to grant the permissions using the request headers:   Specify a canned ACL with the x-amz-acl request header. For more information, see Canned ACL.   Specify access permissions explicitly with the x-amz-grant-read, x-amz-grant-read-acp, x-amz-grant-write-acp, and x-amz-grant-full-control headers. These parameters map to the set of permissions that Amazon S3 supports in an ACL. For more information, see Access Control List (ACL) Overview.   You can use either a canned ACL or specify access permissions explicitly. You cannot do both.  Server-Side- Encryption-Specific Request Headers  You can optionally tell Amazon S3 to encrypt data at rest using server-side encryption. Server-side encryption is for data encryption at rest. Amazon S3 encrypts your data as it writes it to disks in its data centers and decrypts it when you access it. The option you use depends on whether you want to use AWS managed encryption keys or provide your own encryption key.    Use encryption keys managed by Amazon S3 or customer master keys (CMKs) stored in AWS Key Management Service (AWS KMS) – If you want AWS to manage the keys used to encrypt data, specify the following headers in the request.   x-amz-server-side-encryption   x-amz-server-side-encryption-aws-kms-key-id   x-amz-server-side-encryption-context    If you specify x-amz-server-side-encryption:aws:kms, but don't provide x-amz-server-side-encryption-aws-kms-key-id, Amazon S3 uses the AWS managed CMK in AWS KMS to protect the data.   All GET and PUT requests for an object protected by AWS KMS fail if you don't make them with SSL or by using SigV4.  For more information about server-side encryption with CMKs stored in AWS KMS (SSE-KMS), see Protecting Data Using Server-Side Encryption with CMKs stored in AWS KMS.   Use customer-provided encryption keys – If you want to manage your own encryption keys, provide all the following headers in the request.   x-amz-server-side-encryption-customer-algorithm   x-amz-server-side-encryption-customer-key   x-amz-server-side-encryption-customer-key-MD5   For more information about server-side encryption with CMKs stored in AWS KMS (SSE-KMS), see Protecting Data Using Server-Side Encryption with CMKs stored in AWS KMS.    Access-Control-List (ACL)-Specific Request Headers  You also can use the following access control–related headers with this operation. By default, all objects are private. Only the owner has full access control. When adding a new object, you can grant permissions to individual AWS accounts or to predefined groups defined by Amazon S3. These permissions are then added to the access control list (ACL) on the object. For more information, see Using ACLs. With this operation, you can grant access permissions using one of the following two methods:   Specify a canned ACL (x-amz-acl) — Amazon S3 supports a set of predefined ACLs, known as canned ACLs. Each canned ACL has a predefined set of grantees and permissions. For more information, see Canned ACL.   Specify access permissions explicitly — To explicitly grant access permissions to specific AWS accounts or groups, use the following headers. Each header maps to specific permissions that Amazon S3 supports in an ACL. For more information, see Access Control List (ACL) Overview. In the header, you specify a list of grantees who get the specific permission. To grant permissions explicitly, use:   x-amz-grant-read   x-amz-grant-write   x-amz-grant-read-acp   x-amz-grant-write-acp   x-amz-grant-full-control   You specify each grantee as a type=value pair, where the type is one of the following:    id – if the value specified is the canonical user ID of an AWS account    uri – if you are granting permissions to a predefined group    emailAddress – if the value specified is the email address of an AWS account  Using email addresses to specify a grantee is only supported in the following AWS Regions:    US East (N. Virginia)   US West (N. California)    US West (Oregon)    Asia Pacific (Singapore)   Asia Pacific (Sydney)   Asia Pacific (Tokyo)   Europe (Ireland)   South America (São Paulo)   For a list of all the Amazon S3 supported Regions and endpoints, see Regions and Endpoints in the AWS General Reference.    For example, the following x-amz-grant-read header grants the AWS accounts identified by account IDs permissions to read object data and its metadata:  x-amz-grant-read: id="11112222333", id="444455556666"       The following operations are related to CreateMultipartUpload:    UploadPart     CompleteMultipartUpload     AbortMultipartUpload     ListParts     ListMultipartUploads

    Args:
        acl(str, optional): The canned ACL to apply to the object. This action is not supported by Amazon S3 on Outposts. Defaults to None.
        bucket(str): The name of the bucket to which to initiate the upload When using this action with an access point, you must direct requests to the access point hostname. The access point hostname takes the form AccessPointName-AccountId.s3-accesspoint.Region.amazonaws.com. When using this action with an access point through the AWS SDKs, you provide the access point ARN in place of the bucket name. For more information about access point ARNs, see Using access points in the Amazon S3 User Guide. When using this action with Amazon S3 on Outposts, you must direct requests to the S3 on Outposts hostname. The S3 on Outposts hostname takes the form AccessPointName-AccountId.outpostID.s3-outposts.Region.amazonaws.com. When using this action using S3 on Outposts through the AWS SDKs, you provide the Outposts bucket ARN in place of the bucket name. For more information about S3 on Outposts ARNs, see Using S3 on Outposts in the Amazon S3 User Guide.
        cache_control(str, optional): Specifies caching behavior along the request/reply chain. Defaults to None.
        content_disposition(str, optional): Specifies presentational information for the object. Defaults to None.
        content_encoding(str, optional): Specifies what content encodings have been applied to the object and thus what decoding mechanisms must be applied to obtain the media-type referenced by the Content-Type header field. Defaults to None.
        content_language(str, optional): The language the content is in. Defaults to None.
        content_type(str, optional): A standard MIME type describing the format of the object data. Defaults to None.
        expires(str, optional): The date and time at which the object is no longer cacheable. Defaults to None.
        grant_full_control(str, optional): Gives the grantee READ, READ_ACP, and WRITE_ACP permissions on the object. This action is not supported by Amazon S3 on Outposts. Defaults to None.
        grant_read(str, optional): Allows grantee to read the object data and its metadata. This action is not supported by Amazon S3 on Outposts. Defaults to None.
        grant_read_acp(str, optional): Allows grantee to read the object ACL. This action is not supported by Amazon S3 on Outposts. Defaults to None.
        grant_write_acp(str, optional): Allows grantee to write the ACL for the applicable object. This action is not supported by Amazon S3 on Outposts. Defaults to None.
        key(str): Object key for which the multipart upload is to be initiated.
        metadata(Dict, optional): A map of metadata to store with the object in S3. Defaults to None.
        server_side_encryption(str, optional): The server-side encryption algorithm used when storing this object in Amazon S3 (for example, AES256, aws:kms). Defaults to None.
        storage_class(str, optional): By default, Amazon S3 uses the STANDARD Storage Class to store newly created objects. The STANDARD storage class provides high durability and high availability. Depending on performance needs, you can specify a different Storage Class. Amazon S3 on Outposts only uses the OUTPOSTS Storage Class. For more information, see Storage Classes in the Amazon S3 User Guide. Defaults to None.
        website_redirect_location(str, optional): If the bucket is configured as a website, redirects requests for this object to another object in the same bucket or to an external URL. Amazon S3 stores the value of this header in the object metadata. Defaults to None.
        sse_customer_algorithm(str, optional): Specifies the algorithm to use to when encrypting the object (for example, AES256). Defaults to None.
        sse_customer_key(str, optional): Specifies the customer-provided encryption key for Amazon S3 to use in encrypting data. This value is used to store the object and then it is discarded; Amazon S3 does not store the encryption key. The key must be appropriate for use with the algorithm specified in the x-amz-server-side-encryption-customer-algorithm header. Defaults to None.
        sse_customer_key_md5(str, optional): Specifies the 128-bit MD5 digest of the encryption key according to RFC 1321. Amazon S3 uses this header for a message integrity check to ensure that the encryption key was transmitted without error. Defaults to None.
        ssekms_key_id(str, optional): Specifies the ID of the symmetric customer managed AWS KMS CMK to use for object encryption. All GET and PUT requests for an object protected by AWS KMS will fail if not made via SSL or using SigV4. For information about configuring using any of the officially supported AWS SDKs and AWS CLI, see Specifying the Signature Version in Request Authentication in the Amazon S3 User Guide. Defaults to None.
        ssekms_encryption_context(str, optional): Specifies the AWS KMS Encryption Context to use for object encryption. The value of this header is a base64-encoded UTF-8 string holding JSON with the encryption context key-value pairs. Defaults to None.
        bucket_key_enabled(bool, optional): Specifies whether Amazon S3 should use an S3 Bucket Key for object encryption with server-side encryption using AWS KMS (SSE-KMS). Setting this header to true causes Amazon S3 to use an S3 Bucket Key for object encryption with SSE-KMS. Specifying this header with an object action doesn’t affect bucket-level settings for S3 Bucket Key. Defaults to None.
        request_payer(str, optional): Confirms that the requester knows that they will be charged for the request. Bucket owners need not specify this parameter in their requests. For information about downloading objects from requester pays buckets, see Downloading Objects in Requestor Pays Buckets in the Amazon S3 User Guide. Defaults to None.
        tagging(str, optional): The tag-set for the object. The tag-set must be encoded as URL Query parameters. Defaults to None.
        object_lock_mode(str, optional): Specifies the Object Lock mode that you want to apply to the uploaded object. Defaults to None.
        object_lock_retain_until_date(str, optional): Specifies the date and time when you want the Object Lock to expire. Defaults to None.
        object_lock_legal_hold_status(str, optional): Specifies whether you want to apply a Legal Hold to the uploaded object. Defaults to None.
        expected_bucket_owner(str, optional): The account ID of the expected bucket owner. If the bucket is owned by a different account, the request will fail with an HTTP 403 (Access Denied) error. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.s3.multipart_upload.create(ctx, bucket=value, key=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.s3.multipart_upload.create bucket=value, key=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="s3",
        operation="create_multipart_upload",
        op_kwargs={
            "ACL": acl,
            "Bucket": bucket,
            "CacheControl": cache_control,
            "ContentDisposition": content_disposition,
            "ContentEncoding": content_encoding,
            "ContentLanguage": content_language,
            "ContentType": content_type,
            "Expires": expires,
            "GrantFullControl": grant_full_control,
            "GrantRead": grant_read,
            "GrantReadACP": grant_read_acp,
            "GrantWriteACP": grant_write_acp,
            "Key": key,
            "Metadata": metadata,
            "ServerSideEncryption": server_side_encryption,
            "StorageClass": storage_class,
            "WebsiteRedirectLocation": website_redirect_location,
            "SSECustomerAlgorithm": sse_customer_algorithm,
            "SSECustomerKey": sse_customer_key,
            "SSECustomerKeyMD5": sse_customer_key_md5,
            "SSEKMSKeyId": ssekms_key_id,
            "SSEKMSEncryptionContext": ssekms_encryption_context,
            "BucketKeyEnabled": bucket_key_enabled,
            "RequestPayer": request_payer,
            "Tagging": tagging,
            "ObjectLockMode": object_lock_mode,
            "ObjectLockRetainUntilDate": object_lock_retain_until_date,
            "ObjectLockLegalHoldStatus": object_lock_legal_hold_status,
            "ExpectedBucketOwner": expected_bucket_owner,
        },
    )


async def list_all(
    hub,
    ctx,
    bucket: str,
    delimiter: str = None,
    encoding_type: str = None,
    key_marker: str = None,
    max_uploads: int = None,
    prefix: str = None,
    upload_id_marker: str = None,
    expected_bucket_owner: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    This action lists in-progress multipart uploads. An in-progress multipart upload is a multipart upload that has been initiated using the Initiate Multipart Upload request, but has not yet been completed or aborted. This action returns at most 1,000 multipart uploads in the response. 1,000 multipart uploads is the maximum number of uploads a response can include, which is also the default value. You can further limit the number of uploads in a response by specifying the max-uploads parameter in the response. If additional multipart uploads satisfy the list criteria, the response will contain an IsTruncated element with the value true. To list the additional multipart uploads, use the key-marker and upload-id-marker request parameters. In the response, the uploads are sorted by key. If your application has initiated more than one multipart upload using the same object key, then uploads in the response are first sorted by key. Additionally, uploads are sorted in ascending order within each key by the upload initiation time. For more information on multipart uploads, see Uploading Objects Using Multipart Upload. For information on permissions required to use the multipart upload API, see Multipart Upload and Permissions. The following operations are related to ListMultipartUploads:    CreateMultipartUpload     UploadPart     CompleteMultipartUpload     ListParts     AbortMultipartUpload

    Args:
        bucket(str): The name of the bucket to which the multipart upload was initiated.  When using this action with an access point, you must direct requests to the access point hostname. The access point hostname takes the form AccessPointName-AccountId.s3-accesspoint.Region.amazonaws.com. When using this action with an access point through the AWS SDKs, you provide the access point ARN in place of the bucket name. For more information about access point ARNs, see Using access points in the Amazon S3 User Guide. When using this action with Amazon S3 on Outposts, you must direct requests to the S3 on Outposts hostname. The S3 on Outposts hostname takes the form AccessPointName-AccountId.outpostID.s3-outposts.Region.amazonaws.com. When using this action using S3 on Outposts through the AWS SDKs, you provide the Outposts bucket ARN in place of the bucket name. For more information about S3 on Outposts ARNs, see Using S3 on Outposts in the Amazon S3 User Guide.
        delimiter(str, optional): Character you use to group keys. All keys that contain the same string between the prefix, if specified, and the first occurrence of the delimiter after the prefix are grouped under a single result element, CommonPrefixes. If you don't specify the prefix parameter, then the substring starts at the beginning of the key. The keys that are grouped under CommonPrefixes result element are not returned elsewhere in the response. Defaults to None.
        encoding_type(str, optional): Requests Amazon S3 to encode the object keys in the response and specifies the encoding method to use. An object key may contain any Unicode character; however, XML 1.0 parser cannot parse some characters, such as characters with an ASCII value from 0 to 10. For characters that are not supported in XML 1.0, you can add this parameter to request that Amazon S3 encode the keys in the response. Defaults to None.
        key_marker(str, optional): Together with upload-id-marker, this parameter specifies the multipart upload after which listing should begin. If upload-id-marker is not specified, only the keys lexicographically greater than the specified key-marker will be included in the list. If upload-id-marker is specified, any multipart uploads for a key equal to the key-marker might also be included, provided those multipart uploads have upload IDs lexicographically greater than the specified upload-id-marker. Defaults to None.
        max_uploads(int, optional): Sets the maximum number of multipart uploads, from 1 to 1,000, to return in the response body. 1,000 is the maximum number of uploads that can be returned in a response. Defaults to None.
        prefix(str, optional): Lists in-progress uploads only for those keys that begin with the specified prefix. You can use prefixes to separate a bucket into different grouping of keys. (You can think of using prefix to make groups in the same way you'd use a folder in a file system.). Defaults to None.
        upload_id_marker(str, optional): Together with key-marker, specifies the multipart upload after which listing should begin. If key-marker is not specified, the upload-id-marker parameter is ignored. Otherwise, any multipart uploads for a key equal to the key-marker might be included in the list only if they have an upload ID lexicographically greater than the specified upload-id-marker. Defaults to None.
        expected_bucket_owner(str, optional): The account ID of the expected bucket owner. If the bucket is owned by a different account, the request will fail with an HTTP 403 (Access Denied) error. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.s3.multipart_upload.list_all(ctx, bucket=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.s3.multipart_upload.list_all bucket=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="s3",
        operation="list_multipart_uploads",
        op_kwargs={
            "Bucket": bucket,
            "Delimiter": delimiter,
            "EncodingType": encoding_type,
            "KeyMarker": key_marker,
            "MaxUploads": max_uploads,
            "Prefix": prefix,
            "UploadIdMarker": upload_id_marker,
            "ExpectedBucketOwner": expected_bucket_owner,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def tag(hub, ctx, resource_arn: str, tags: Dict) -> Dict:
    r"""
    **Autogenerated function**

     Use to assign one or more tags to a resource.

    Args:
        resource_arn(str):  The Amazon Resource Name (ARN) of the resource that the tags are added to. .
        tags(Dict):  The list of tags that are added to the specified resource. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codeguruprofiler.resource.tag(
                ctx, resource_arn=value, tags=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codeguruprofiler.resource.tag resource_arn=value, tags=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codeguruprofiler",
        operation="tag_resource",
        op_kwargs={"resourceArn": resource_arn, "tags": tags},
    )


async def untag(hub, ctx, resource_arn: str, tag_keys: List) -> Dict:
    r"""
    **Autogenerated function**

     Use to remove one or more tags from a resource.

    Args:
        resource_arn(str):  The Amazon Resource Name (ARN) of the resource that contains the tags to remove. .
        tag_keys(List):  A list of tag keys. Existing tags of resources with keys in this list are removed from the specified resource. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codeguruprofiler.resource.untag(
                ctx, resource_arn=value, tag_keys=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codeguruprofiler.resource.untag resource_arn=value, tag_keys=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codeguruprofiler",
        operation="untag_resource",
        op_kwargs={"resourceArn": resource_arn, "tagKeys": tag_keys},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def delete_multiple(hub, ctx, identity_ids_to_delete: List) -> Dict:
    r"""
    **Autogenerated function**

    Deletes identities from an identity pool. You can specify a list of 1-60 identities that you want to delete. You must use AWS Developer credentials to call this API.

    Args:
        identity_ids_to_delete(List): A list of 1-60 identities that you want to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cognito_identity.identity.init.delete_multiple(
                ctx, identity_ids_to_delete=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cognito_identity.identity.init.delete_multiple identity_ids_to_delete=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cognito-identity",
        operation="delete_identities",
        op_kwargs={"IdentityIdsToDelete": identity_ids_to_delete},
    )


async def describe(hub, ctx, identity_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns metadata related to the given identity, including when the identity was created and any associated linked logins. You must use AWS Developer credentials to call this API.

    Args:
        identity_id(str): A unique identifier in the format REGION:GUID.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cognito_identity.identity.init.describe(ctx, identity_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cognito_identity.identity.init.describe identity_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cognito-identity",
        operation="describe_identity",
        op_kwargs={"IdentityId": identity_id},
    )


async def list_all(
    hub,
    ctx,
    identity_pool_id: str,
    max_results: int,
    next_token: str = None,
    hide_disabled: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the identities in an identity pool. You must use AWS Developer credentials to call this API.

    Args:
        identity_pool_id(str): An identity pool ID in the format REGION:GUID.
        max_results(int): The maximum number of identities to return.
        next_token(str, optional): A pagination token. Defaults to None.
        hide_disabled(bool, optional): An optional boolean parameter that allows you to hide disabled identities. If omitted, the ListIdentities API will include disabled identities in the response. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cognito_identity.identity.init.list_all(
                ctx, identity_pool_id=value, max_results=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cognito_identity.identity.init.list_all identity_pool_id=value, max_results=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cognito-identity",
        operation="list_identities",
        op_kwargs={
            "IdentityPoolId": identity_pool_id,
            "MaxResults": max_results,
            "NextToken": next_token,
            "HideDisabled": hide_disabled,
        },
    )


async def unlink(
    hub, ctx, identity_id: str, logins: Dict, logins_to_remove: List
) -> None:
    r"""
    **Autogenerated function**

    Unlinks a federated identity from an existing account. Unlinked logins will be considered new identities next time they are seen. Removing the last linked login will make this identity inaccessible. This is a public API. You do not need any credentials to call this API.

    Args:
        identity_id(str): A unique identifier in the format REGION:GUID.
        logins(Dict): A set of optional name-value pairs that map provider names to provider tokens.
        logins_to_remove(List): Provider names to unlink from this identity.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cognito_identity.identity.init.unlink(
                ctx, identity_id=value, logins=value, logins_to_remove=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cognito_identity.identity.init.unlink identity_id=value, logins=value, logins_to_remove=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cognito-identity",
        operation="unlink_identity",
        op_kwargs={
            "IdentityId": identity_id,
            "Logins": logins,
            "LoginsToRemove": logins_to_remove,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(hub, ctx, key_group_config: Dict) -> Dict:
    r"""
    **Autogenerated function**

    Creates a key group that you can use with CloudFront signed URLs and signed cookies. To create a key group, you must specify at least one public key for the key group. After you create a key group, you can reference it from one or more cache behaviors. When you reference a key group in a cache behavior, CloudFront requires signed URLs or signed cookies for all requests that match the cache behavior. The URLs or cookies must be signed with a private key whose corresponding public key is in the key group. The signed URL or cookie contains information about which public key CloudFront should use to verify the signature. For more information, see Serving private content in the Amazon CloudFront Developer Guide.

    Args:
        key_group_config(Dict): A key group configuration.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.key_group.init.create(ctx, key_group_config=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.key_group.init.create key_group_config=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="create_key_group",
        op_kwargs={"KeyGroupConfig": key_group_config},
    )


async def delete(hub, ctx, id_: str, if_match: str = None) -> None:
    r"""
    **Autogenerated function**

    Deletes a key group. You cannot delete a key group that is referenced in a cache behavior. First update your distributions to remove the key group from all cache behaviors, then delete the key group. To delete a key group, you must provide the key group’s identifier and version. To get these values, use ListKeyGroups followed by GetKeyGroup or GetKeyGroupConfig.

    Args:
        id_(str): The identifier of the key group that you are deleting. To get the identifier, use ListKeyGroups.
        if_match(str, optional): The version of the key group that you are deleting. The version is the key group’s ETag value. To get the ETag, use GetKeyGroup or GetKeyGroupConfig. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.key_group.init.delete(ctx, id_=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.key_group.init.delete id_=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="delete_key_group",
        op_kwargs={"Id": id_, "IfMatch": if_match},
    )


async def get(hub, ctx, id_: str) -> Dict:
    r"""
    **Autogenerated function**

    Gets a key group, including the date and time when the key group was last modified. To get a key group, you must provide the key group’s identifier. If the key group is referenced in a distribution’s cache behavior, you can get the key group’s identifier using ListDistributions or GetDistribution. If the key group is not referenced in a cache behavior, you can get the identifier using ListKeyGroups.

    Args:
        id_(str): The identifier of the key group that you are getting. To get the identifier, use ListKeyGroups.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.key_group.init.get(ctx, id_=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.key_group.init.get id_=value
    """
    return await hub.tool.aws.client.exec(
        ctx, service_name="cloudfront", operation="get_key_group", op_kwargs={"Id": id_}
    )


async def list_all(hub, ctx, marker: str = None, max_items: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Gets a list of key groups. You can optionally specify the maximum number of items to receive in the response. If the total number of items in the list exceeds the maximum that you specify, or the default maximum, the response is paginated. To get the next page of items, send a subsequent request that specifies the NextMarker value from the current response as the Marker value in the subsequent request.

    Args:
        marker(str, optional): Use this field when paginating results to indicate where to begin in your list of key groups. The response includes key groups in the list that occur after the marker. To get the next page of the list, set this field’s value to the value of NextMarker from the current page’s response. Defaults to None.
        max_items(str, optional): The maximum number of key groups that you want in the response. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.key_group.init.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.key_group.init.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="list_key_groups",
        op_kwargs={"Marker": marker, "MaxItems": max_items},
    )


async def update(
    hub, ctx, key_group_config: Dict, id_: str, if_match: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Updates a key group. When you update a key group, all the fields are updated with the values provided in the request. You cannot update some fields independent of others. To update a key group:   Get the current key group with GetKeyGroup or GetKeyGroupConfig.   Locally modify the fields in the key group that you want to update. For example, add or remove public key IDs.   Call UpdateKeyGroup with the entire key group object, including the fields that you modified and those that you didn’t.

    Args:
        key_group_config(Dict): The key group configuration.
        id_(str): The identifier of the key group that you are updating.
        if_match(str, optional): The version of the key group that you are updating. The version is the key group’s ETag value. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.key_group.init.update(
                ctx, key_group_config=value, id_=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.key_group.init.update key_group_config=value, id_=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="update_key_group",
        op_kwargs={"KeyGroupConfig": key_group_config, "Id": id_, "IfMatch": if_match},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def describe_all(
    hub,
    ctx,
    availability_zone: str = None,
    filters: List = None,
    include_marketplace: bool = None,
    instance_type: str = None,
    max_duration: int = None,
    max_instance_count: int = None,
    min_duration: int = None,
    offering_class: str = None,
    product_description: str = None,
    reserved_instances_offering_ids: List = None,
    dry_run: bool = None,
    instance_tenancy: str = None,
    max_results: int = None,
    next_token: str = None,
    offering_type: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Describes Reserved Instance offerings that are available for purchase. With Reserved Instances, you purchase the right to launch instances for a period of time. During that time period, you do not receive insufficient capacity errors, and you pay a lower usage rate than the rate charged for On-Demand instances for the actual time used. If you have listed your own Reserved Instances for sale in the Reserved Instance Marketplace, they will be excluded from these results. This is to ensure that you do not purchase your own Reserved Instances. For more information, see Reserved Instance Marketplace in the Amazon EC2 User Guide.

    Args:
        availability_zone(str, optional): The Availability Zone in which the Reserved Instance can be used. Defaults to None.
        filters(List, optional): One or more filters.    availability-zone - The Availability Zone where the Reserved Instance can be used.    duration - The duration of the Reserved Instance (for example, one year or three years), in seconds (31536000 | 94608000).    fixed-price - The purchase price of the Reserved Instance (for example, 9800.0).    instance-type - The instance type that is covered by the reservation.    marketplace - Set to true to show only Reserved Instance Marketplace offerings. When this filter is not used, which is the default behavior, all offerings from both Amazon Web Services and the Reserved Instance Marketplace are listed.    product-description - The Reserved Instance product platform description. Instances that include (Amazon VPC) in the product platform description will only be displayed to EC2-Classic account holders and are for use with Amazon VPC. (Linux/UNIX | Linux/UNIX (Amazon VPC) | SUSE Linux | SUSE Linux (Amazon VPC) | Red Hat Enterprise Linux | Red Hat Enterprise Linux (Amazon VPC) | Red Hat Enterprise Linux with HA (Amazon VPC) | Windows | Windows (Amazon VPC) | Windows with SQL Server Standard | Windows with SQL Server Standard (Amazon VPC) | Windows with SQL Server Web |  Windows with SQL Server Web (Amazon VPC) | Windows with SQL Server Enterprise | Windows with SQL Server Enterprise (Amazon VPC))     reserved-instances-offering-id - The Reserved Instances offering ID.    scope - The scope of the Reserved Instance (Availability Zone or Region).    usage-price - The usage price of the Reserved Instance, per hour (for example, 0.84).  . Defaults to None.
        include_marketplace(bool, optional): Include Reserved Instance Marketplace offerings in the response. Defaults to None.
        instance_type(str, optional): The instance type that the reservation will cover (for example, m1.small). For more information, see Instance types in the Amazon EC2 User Guide. Defaults to None.
        max_duration(int, optional): The maximum duration (in seconds) to filter when searching for offerings. Default: 94608000 (3 years). Defaults to None.
        max_instance_count(int, optional): The maximum number of instances to filter when searching for offerings. Default: 20. Defaults to None.
        min_duration(int, optional): The minimum duration (in seconds) to filter when searching for offerings. Default: 2592000 (1 month). Defaults to None.
        offering_class(str, optional): The offering class of the Reserved Instance. Can be standard or convertible. Defaults to None.
        product_description(str, optional): The Reserved Instance product platform description. Instances that include (Amazon VPC) in the description are for use with Amazon VPC. Defaults to None.
        reserved_instances_offering_ids(List, optional): One or more Reserved Instances offering IDs. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        instance_tenancy(str, optional): The tenancy of the instances covered by the reservation. A Reserved Instance with a tenancy of dedicated is applied to instances that run in a VPC on single-tenant hardware (i.e., Dedicated Instances).  Important: The host value cannot be used with this parameter. Use the default or dedicated values only. Default: default . Defaults to None.
        max_results(int, optional): The maximum number of results to return for the request in a single page. The remaining results of the initial request can be seen by sending another request with the returned NextToken value. The maximum is 100. Default: 100. Defaults to None.
        next_token(str, optional): The token to retrieve the next page of results. Defaults to None.
        offering_type(str, optional): The Reserved Instance offering type. If you are using tools that predate the 2011-11-01 API version, you only have access to the Medium Utilization Reserved Instance offering type. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.reserved_instance.offering.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.reserved_instance.offering.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="describe_reserved_instances_offerings",
        op_kwargs={
            "AvailabilityZone": availability_zone,
            "Filters": filters,
            "IncludeMarketplace": include_marketplace,
            "InstanceType": instance_type,
            "MaxDuration": max_duration,
            "MaxInstanceCount": max_instance_count,
            "MinDuration": min_duration,
            "OfferingClass": offering_class,
            "ProductDescription": product_description,
            "ReservedInstancesOfferingIds": reserved_instances_offering_ids,
            "DryRun": dry_run,
            "InstanceTenancy": instance_tenancy,
            "MaxResults": max_results,
            "NextToken": next_token,
            "OfferingType": offering_type,
        },
    )


async def purchase(
    hub,
    ctx,
    instance_count: int,
    reserved_instances_offering_id: str,
    dry_run: bool = None,
    limit_price: Dict = None,
    purchase_time: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Purchases a Reserved Instance for use with your account. With Reserved Instances, you pay a lower hourly rate compared to On-Demand instance pricing. Use DescribeReservedInstancesOfferings to get a list of Reserved Instance offerings that match your specifications. After you've purchased a Reserved Instance, you can check for your new Reserved Instance with DescribeReservedInstances. To queue a purchase for a future date and time, specify a purchase time. If you do not specify a purchase time, the default is the current time. For more information, see Reserved Instances and Reserved Instance Marketplace in the Amazon EC2 User Guide.

    Args:
        instance_count(int): The number of Reserved Instances to purchase.
        reserved_instances_offering_id(str): The ID of the Reserved Instance offering to purchase.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        limit_price(Dict, optional): Specified for Reserved Instance Marketplace offerings to limit the total order and ensure that the Reserved Instances are not purchased at unexpected prices. Defaults to None.
        purchase_time(str, optional): The time at which to purchase the Reserved Instance, in UTC format (for example, YYYY-MM-DDTHH:MM:SSZ). Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.reserved_instance.offering.purchase(
                ctx, instance_count=value, reserved_instances_offering_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.reserved_instance.offering.purchase instance_count=value, reserved_instances_offering_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="purchase_reserved_instances_offering",
        op_kwargs={
            "InstanceCount": instance_count,
            "ReservedInstancesOfferingId": reserved_instances_offering_id,
            "DryRun": dry_run,
            "LimitPrice": limit_price,
            "PurchaseTime": purchase_time,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def update(
    hub, ctx, old_approval_rule_template_name: str, new_approval_rule_template_name: str
) -> Dict:
    r"""
    **Autogenerated function**

    Updates the name of a specified approval rule template.

    Args:
        old_approval_rule_template_name(str): The current name of the approval rule template.
        new_approval_rule_template_name(str): The new name you want to apply to the approval rule template.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codecommit.approval_rule_template.name.update(
                ctx, old_approval_rule_template_name=value, new_approval_rule_template_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codecommit.approval_rule_template.name.update old_approval_rule_template_name=value, new_approval_rule_template_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codecommit",
        operation="update_approval_rule_template_name",
        op_kwargs={
            "oldApprovalRuleTemplateName": old_approval_rule_template_name,
            "newApprovalRuleTemplateName": new_approval_rule_template_name,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx, message_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves the raw content of an in-transit email message, in MIME format.

    Args:
        message_id(str): The identifier of the email message to retrieve.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.workmailmessageflow_raw_message_content.get(
                ctx, message_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.workmailmessageflow_raw_message_content.get message_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="workmailmessageflow",
        operation="get_raw_message_content",
        op_kwargs={"messageId": message_id},
    )


async def put(hub, ctx, message_id: str, content: Dict) -> Dict:
    r"""
    **Autogenerated function**

    Updates the raw content of an in-transit email message, in MIME format. This example describes how to update in-transit email message. For more information and examples for using this API, see  Updating message content with AWS Lambda.  Updates to an in-transit message only appear when you call PutRawMessageContent from an AWS Lambda function configured with a synchronous  Run Lambda rule. If you call PutRawMessageContent on a delivered or sent message, the message remains unchanged, even though GetRawMessageContent returns an updated message.

    Args:
        message_id(str): The identifier of the email message being updated.
        content(Dict): Describes the raw message content of the updated email message.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.workmailmessageflow_raw_message_content.put(
                ctx, message_id=value, content=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.workmailmessageflow_raw_message_content.put message_id=value, content=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="workmailmessageflow",
        operation="put_raw_message_content",
        op_kwargs={"messageId": message_id, "content": content},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    transit_gateway_route_table_id: str,
    prefix_list_id: str,
    transit_gateway_attachment_id: str = None,
    blackhole: bool = None,
    dry_run: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a reference (route) to a prefix list in a specified transit gateway route table.

    Args:
        transit_gateway_route_table_id(str): The ID of the transit gateway route table.
        prefix_list_id(str): The ID of the prefix list that is used for destination matches.
        transit_gateway_attachment_id(str, optional): The ID of the attachment to which traffic is routed. Defaults to None.
        blackhole(bool, optional): Indicates whether to drop traffic that matches this route. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.transit_gateway.prefix_list_reference.create(
                ctx, transit_gateway_route_table_id=value, prefix_list_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.transit_gateway.prefix_list_reference.create transit_gateway_route_table_id=value, prefix_list_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="create_transit_gateway_prefix_list_reference",
        op_kwargs={
            "TransitGatewayRouteTableId": transit_gateway_route_table_id,
            "PrefixListId": prefix_list_id,
            "TransitGatewayAttachmentId": transit_gateway_attachment_id,
            "Blackhole": blackhole,
            "DryRun": dry_run,
        },
    )


async def delete(
    hub,
    ctx,
    transit_gateway_route_table_id: str,
    prefix_list_id: str,
    dry_run: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a reference (route) to a prefix list in a specified transit gateway route table.

    Args:
        transit_gateway_route_table_id(str): The ID of the route table.
        prefix_list_id(str): The ID of the prefix list.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.transit_gateway.prefix_list_reference.delete(
                ctx, transit_gateway_route_table_id=value, prefix_list_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.transit_gateway.prefix_list_reference.delete transit_gateway_route_table_id=value, prefix_list_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="delete_transit_gateway_prefix_list_reference",
        op_kwargs={
            "TransitGatewayRouteTableId": transit_gateway_route_table_id,
            "PrefixListId": prefix_list_id,
            "DryRun": dry_run,
        },
    )


async def get_all(
    hub,
    ctx,
    transit_gateway_route_table_id: str,
    filters: List = None,
    max_results: int = None,
    next_token: str = None,
    dry_run: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Gets information about the prefix list references in a specified transit gateway route table.

    Args:
        transit_gateway_route_table_id(str): The ID of the transit gateway route table.
        filters(List, optional): One or more filters. The possible values are:    attachment.resource-id - The ID of the resource for the attachment.    attachment.resource-type - The type of resource for the attachment. Valid values are vpc | vpn | direct-connect-gateway | peering.    attachment.transit-gateway-attachment-id - The ID of the attachment.    is-blackhole - Whether traffic matching the route is blocked (true | false).    prefix-list-id - The ID of the prefix list.    prefix-list-owner-id - The ID of the owner of the prefix list.    state - The state of the prefix list reference (pending | available | modifying | deleting).  . Defaults to None.
        max_results(int, optional): The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned nextToken value. Defaults to None.
        next_token(str, optional): The token for the next page of results. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.transit_gateway.prefix_list_reference.get_all(
                ctx, transit_gateway_route_table_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.transit_gateway.prefix_list_reference.get_all transit_gateway_route_table_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="get_transit_gateway_prefix_list_references",
        op_kwargs={
            "TransitGatewayRouteTableId": transit_gateway_route_table_id,
            "Filters": filters,
            "MaxResults": max_results,
            "NextToken": next_token,
            "DryRun": dry_run,
        },
    )


async def modify(
    hub,
    ctx,
    transit_gateway_route_table_id: str,
    prefix_list_id: str,
    transit_gateway_attachment_id: str = None,
    blackhole: bool = None,
    dry_run: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Modifies a reference (route) to a prefix list in a specified transit gateway route table.

    Args:
        transit_gateway_route_table_id(str): The ID of the transit gateway route table.
        prefix_list_id(str): The ID of the prefix list.
        transit_gateway_attachment_id(str, optional): The ID of the attachment to which traffic is routed. Defaults to None.
        blackhole(bool, optional): Indicates whether to drop traffic that matches this route. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.transit_gateway.prefix_list_reference.modify(
                ctx, transit_gateway_route_table_id=value, prefix_list_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.transit_gateway.prefix_list_reference.modify transit_gateway_route_table_id=value, prefix_list_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="modify_transit_gateway_prefix_list_reference",
        op_kwargs={
            "TransitGatewayRouteTableId": transit_gateway_route_table_id,
            "PrefixListId": prefix_list_id,
            "TransitGatewayAttachmentId": transit_gateway_attachment_id,
            "Blackhole": blackhole,
            "DryRun": dry_run,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx, operating_system: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves the default patch baseline. Amazon Web Services Systems Manager supports creating multiple default patch baselines. For example, you can create a default patch baseline for each operating system. If you don't specify an operating system value, the default patch baseline for Windows is returned.

    Args:
        operating_system(str, optional): Returns the default patch baseline for the specified operating system. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.default_patch_baseline.get(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.default_patch_baseline.get
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="get_default_patch_baseline",
        op_kwargs={"OperatingSystem": operating_system},
    )


async def register(hub, ctx, baseline_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Defines the default patch baseline for the relevant operating system. To reset the Amazon Web Services-predefined patch baseline as the default, specify the full patch baseline Amazon Resource Name (ARN) as the baseline ID value. For example, for CentOS, specify arn:aws:ssm:us-east-2:733109147000:patchbaseline/pb-0574b43a65ea646ed instead of pb-0574b43a65ea646ed.

    Args:
        baseline_id(str): The ID of the patch baseline that should be the default patch baseline.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.default_patch_baseline.register(ctx, baseline_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.default_patch_baseline.register baseline_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="register_default_patch_baseline",
        op_kwargs={"BaselineId": baseline_id},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def list_all(hub, ctx, next_token: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Returns a list of all historical purchases, renewals, and system renewal transactions for an AWS account. The list is paginated and ordered by a descending timestamp (most recent transactions are first). The API returns a NotEligible error if the user is not permitted to invoke the operation. If you must be able to invoke this operation, contact aws-devicefarm-support@amazon.com.

    Args:
        next_token(str, optional): An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.devicefarm.offering.transaction.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.devicefarm.offering.transaction.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="devicefarm",
        operation="list_offering_transactions",
        op_kwargs={"nextToken": next_token},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def reboot(hub, ctx, broker_ids: List, cluster_arn: str) -> Dict:
    r"""
        **Autogenerated function**

        Reboots brokers.

        Args:
            broker_ids(List):
    The list of broker IDs to be rebooted. The reboot-broker operation supports rebooting one broker at a time.
    .
            cluster_arn(str):
    The Amazon Resource Name (ARN) of the cluster to be updated.
    .

        Returns:
            Dict

        Examples:
            Call from code:

            .. code-block:: python

                await hub.exec.aws_auto.kafka.broker.init.reboot(
                    ctx, broker_ids=value, cluster_arn=value
                )

            Call from CLI:

            .. code-block:: bash

                $ idem exec aws_auto.kafka.broker.init.reboot broker_ids=value, cluster_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="kafka",
        operation="reboot_broker",
        op_kwargs={"BrokerIds": broker_ids, "ClusterArn": cluster_arn},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx, insight_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves the summary information of an insight. This includes impact to clients and root cause services, the top anomalous services, the category, the state of the insight, and the start and end time of the insight.

    Args:
        insight_id(str): The insight's unique identifier. Use the GetInsightSummaries action to retrieve an InsightId.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.xray.insight.init.get(ctx, insight_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.xray.insight.init.get insight_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="xray",
        operation="get_insight",
        op_kwargs={"InsightId": insight_id},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def delete(hub, ctx, query_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes the stored query for a single AWS account and a single AWS Region.

    Args:
        query_name(str): The name of the query that you want to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.config.stored_query.delete(ctx, query_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.config.stored_query.delete query_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="config",
        operation="delete_stored_query",
        op_kwargs={"QueryName": query_name},
    )


async def get(hub, ctx, query_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns the details of a specific stored query.

    Args:
        query_name(str): The name of the query.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.config.stored_query.get(ctx, query_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.config.stored_query.get query_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="config",
        operation="get_stored_query",
        op_kwargs={"QueryName": query_name},
    )


async def list_all(hub, ctx, next_token: str = None, max_results: int = None) -> Dict:
    r"""
    **Autogenerated function**

    Lists the stored queries for a single AWS account and a single AWS Region. The default is 100.

    Args:
        next_token(str, optional): The nextToken string returned in a previous request that you use to request the next page of results in a paginated response. Defaults to None.
        max_results(int, optional): The maximum number of results to be returned with a single call. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.config.stored_query.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.config.stored_query.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="config",
        operation="list_stored_queries",
        op_kwargs={"NextToken": next_token, "MaxResults": max_results},
    )


async def put(hub, ctx, stored_query: Dict, tags: List = None) -> Dict:
    r"""
    **Autogenerated function**

    Saves a new query or updates an existing saved query. The QueryName must be unique for a single AWS account and a single AWS Region. You can create upto 300 queries in a single AWS account and a single AWS Region.

    Args:
        stored_query(Dict): A list of StoredQuery objects. The mandatory fields are QueryName and Expression.  When you are creating a query, you must provide a query name and an expression. When you are updating a query, you must provide a query name but updating the description is optional. .
        tags(List, optional): A list of Tags object. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.config.stored_query.put(ctx, stored_query=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.config.stored_query.put stored_query=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="config",
        operation="put_stored_query",
        op_kwargs={"StoredQuery": stored_query, "Tags": tags},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def poll(hub, ctx, domain: str, task_list: Dict, identity: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Used by workers to get an ActivityTask from the specified activity taskList. This initiates a long poll, where the service holds the HTTP connection open and responds as soon as a task becomes available. The maximum time the service holds on to the request before responding is 60 seconds. If no task is available within 60 seconds, the poll returns an empty result. An empty result, in this context, means that an ActivityTask is returned, but that the value of taskToken is an empty string. If a task is returned, the worker should use its type to identify and process it correctly.  Workers should set their client side socket timeout to at least 70 seconds (10 seconds higher than the maximum time service may hold the poll request).   Access Control  You can use IAM policies to control this action's access to Amazon SWF resources as follows:   Use a Resource element with the domain name to limit the action to only specified domains.   Use an Action element to allow or deny permission to call this action.   Constrain the taskList.name parameter by using a Condition element with the swf:taskList.name key to allow the action to access only certain task lists.   If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's cause parameter is set to OPERATION_NOT_PERMITTED. For details and example IAM policies, see Using IAM to Manage Access to Amazon SWF Workflows in the Amazon SWF Developer Guide.

    Args:
        domain(str): The name of the domain that contains the task lists being polled.
        task_list(Dict): Specifies the task list to poll for activity tasks. The specified string must not start or end with whitespace. It must not contain a : (colon), / (slash), | (vertical bar), or any control characters (\u0000-\u001f | \u007f-\u009f). Also, it must not be the literal string arn.
        identity(str, optional): Identity of the worker making the request, recorded in the ActivityTaskStarted event in the workflow history. This enables diagnostic tracing when problems arise. The form of this identity is user defined. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.swf.for_.activity_task.poll(ctx, domain=value, task_list=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.swf.for_.activity_task.poll domain=value, task_list=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="swf",
        operation="poll_for_activity_task",
        op_kwargs={"domain": domain, "taskList": task_list, "identity": identity},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe(hub, ctx, cluster_identifier: str) -> Dict:
    r"""
    **Autogenerated function**

    Describes whether information, such as queries and connection attempts, is being logged for the specified Amazon Redshift cluster.

    Args:
        cluster_identifier(str): The identifier of the cluster from which to get the logging status. Example: examplecluster .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.redshift.logging.status.describe(ctx, cluster_identifier=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.redshift.logging.status.describe cluster_identifier=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="redshift",
        operation="describe_logging_status",
        op_kwargs={"ClusterIdentifier": cluster_identifier},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""


async def disassociate(
    hub, ctx, approval_rule_template_name: str, repository_name: str
) -> None:
    r"""
    **Autogenerated function**

    Removes the association between a template and a repository so that approval rules based on the template are not automatically created when pull requests are created in the specified repository. This does not delete any approval rules previously created for pull requests through the template association.

    Args:
        approval_rule_template_name(str): The name of the approval rule template to disassociate from a specified repository.
        repository_name(str): The name of the repository you want to disassociate from the template.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codecommit.approval_rule_template.from_repository.disassociate(
                ctx, approval_rule_template_name=value, repository_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codecommit.approval_rule_template.from_repository.disassociate approval_rule_template_name=value, repository_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codecommit",
        operation="disassociate_approval_rule_template_from_repository",
        op_kwargs={
            "approvalRuleTemplateName": approval_rule_template_name,
            "repositoryName": repository_name,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def authorize(
    hub,
    ctx,
    cache_security_group_name: str,
    ec2_security_group_name: str,
    ec2_security_group_owner_id: str,
) -> Dict:
    r"""
    **Autogenerated function**

    Allows network ingress to a cache security group. Applications using ElastiCache must be running on Amazon EC2, and Amazon EC2 security groups are used as the authorization mechanism.  You cannot authorize ingress from an Amazon EC2 security group in one region to an ElastiCache cluster in another region.

    Args:
        cache_security_group_name(str): The cache security group that allows network ingress.
        ec2_security_group_name(str): The Amazon EC2 security group to be authorized for ingress to the cache security group.
        ec2_security_group_owner_id(str): The AWS account number of the Amazon EC2 security group owner. Note that this is not the same thing as an AWS access key ID - you must provide a valid AWS account number for this parameter.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.cache.security_group.ingress.authorize(
                ctx,
                cache_security_group_name=value,
                ec2_security_group_name=value,
                ec2_security_group_owner_id=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.cache.security_group.ingress.authorize cache_security_group_name=value, ec2_security_group_name=value, ec2_security_group_owner_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="authorize_cache_security_group_ingress",
        op_kwargs={
            "CacheSecurityGroupName": cache_security_group_name,
            "EC2SecurityGroupName": ec2_security_group_name,
            "EC2SecurityGroupOwnerId": ec2_security_group_owner_id,
        },
    )


async def revoke(
    hub,
    ctx,
    cache_security_group_name: str,
    ec2_security_group_name: str,
    ec2_security_group_owner_id: str,
) -> Dict:
    r"""
    **Autogenerated function**

    Revokes ingress from a cache security group. Use this operation to disallow access from an Amazon EC2 security group that had been previously authorized.

    Args:
        cache_security_group_name(str): The name of the cache security group to revoke ingress from.
        ec2_security_group_name(str): The name of the Amazon EC2 security group to revoke access from.
        ec2_security_group_owner_id(str): The AWS account number of the Amazon EC2 security group owner. Note that this is not the same thing as an AWS access key ID - you must provide a valid AWS account number for this parameter.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.cache.security_group.ingress.revoke(
                ctx,
                cache_security_group_name=value,
                ec2_security_group_name=value,
                ec2_security_group_owner_id=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.cache.security_group.ingress.revoke cache_security_group_name=value, ec2_security_group_name=value, ec2_security_group_owner_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="revoke_cache_security_group_ingress",
        op_kwargs={
            "CacheSecurityGroupName": cache_security_group_name,
            "EC2SecurityGroupName": ec2_security_group_name,
            "EC2SecurityGroupOwnerId": ec2_security_group_owner_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def associate(
    hub, ctx, instance_id: str, resource_type: str, storage_config: Dict
) -> Dict:
    r"""
    **Autogenerated function**

    This API is in preview release for Amazon Connect and is subject to change. Associates a storage resource type for the first time. You can only associate one type of storage configuration in a single call. This means, for example, that you can't define an instance with multiple S3 buckets for storing chat transcripts. This API does not create a resource that doesn't exist. It only associates it to the instance. Ensure that the resource being specified in the storage configuration, like an S3 bucket, exists when being used for association.

    Args:
        instance_id(str): The identifier of the Amazon Connect instance. You can find the instanceId in the ARN of the instance.
        resource_type(str): A valid resource type.
        storage_config(Dict): A valid storage type.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.connect.instance.storage_config.associate(
                ctx, instance_id=value, resource_type=value, storage_config=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.connect.instance.storage_config.associate instance_id=value, resource_type=value, storage_config=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="connect",
        operation="associate_instance_storage_config",
        op_kwargs={
            "InstanceId": instance_id,
            "ResourceType": resource_type,
            "StorageConfig": storage_config,
        },
    )


async def describe(
    hub, ctx, instance_id: str, association_id: str, resource_type: str
) -> Dict:
    r"""
    **Autogenerated function**

    This API is in preview release for Amazon Connect and is subject to change. Retrieves the current storage configurations for the specified resource type, association ID, and instance ID.

    Args:
        instance_id(str): The identifier of the Amazon Connect instance. You can find the instanceId in the ARN of the instance.
        association_id(str): The existing association identifier that uniquely identifies the resource type and storage config for the given instance ID.
        resource_type(str): A valid resource type.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.connect.instance.storage_config.describe(
                ctx, instance_id=value, association_id=value, resource_type=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.connect.instance.storage_config.describe instance_id=value, association_id=value, resource_type=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="connect",
        operation="describe_instance_storage_config",
        op_kwargs={
            "InstanceId": instance_id,
            "AssociationId": association_id,
            "ResourceType": resource_type,
        },
    )


async def disassociate(
    hub, ctx, instance_id: str, association_id: str, resource_type: str
) -> None:
    r"""
    **Autogenerated function**

    This API is in preview release for Amazon Connect and is subject to change. Removes the storage type configurations for the specified resource type and association ID.

    Args:
        instance_id(str): The identifier of the Amazon Connect instance. You can find the instanceId in the ARN of the instance.
        association_id(str): The existing association identifier that uniquely identifies the resource type and storage config for the given instance ID.
        resource_type(str): A valid resource type.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.connect.instance.storage_config.disassociate(
                ctx, instance_id=value, association_id=value, resource_type=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.connect.instance.storage_config.disassociate instance_id=value, association_id=value, resource_type=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="connect",
        operation="disassociate_instance_storage_config",
        op_kwargs={
            "InstanceId": instance_id,
            "AssociationId": association_id,
            "ResourceType": resource_type,
        },
    )


async def list_all(
    hub,
    ctx,
    instance_id: str,
    resource_type: str,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    This API is in preview release for Amazon Connect and is subject to change. Returns a paginated list of storage configs for the identified instance and resource type.

    Args:
        instance_id(str): The identifier of the Amazon Connect instance. You can find the instanceId in the ARN of the instance.
        resource_type(str): A valid resource type.
        next_token(str, optional): The token for the next set of results. Use the value returned in the previous response in the next request to retrieve the next set of results. Defaults to None.
        max_results(int, optional): The maximum number of results to return per page. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.connect.instance.storage_config.list_all(
                ctx, instance_id=value, resource_type=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.connect.instance.storage_config.list_all instance_id=value, resource_type=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="connect",
        operation="list_instance_storage_configs",
        op_kwargs={
            "InstanceId": instance_id,
            "ResourceType": resource_type,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )


async def update(
    hub,
    ctx,
    instance_id: str,
    association_id: str,
    resource_type: str,
    storage_config: Dict,
) -> None:
    r"""
    **Autogenerated function**

    This API is in preview release for Amazon Connect and is subject to change. Updates an existing configuration for a resource type. This API is idempotent.

    Args:
        instance_id(str): The identifier of the Amazon Connect instance. You can find the instanceId in the ARN of the instance.
        association_id(str): The existing association identifier that uniquely identifies the resource type and storage config for the given instance ID.
        resource_type(str): A valid resource type.
        storage_config(Dict): The storage configuration for the instance.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.connect.instance.storage_config.update(
                ctx,
                instance_id=value,
                association_id=value,
                resource_type=value,
                storage_config=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.connect.instance.storage_config.update instance_id=value, association_id=value, resource_type=value, storage_config=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="connect",
        operation="update_instance_storage_config",
        op_kwargs={
            "InstanceId": instance_id,
            "AssociationId": association_id,
            "ResourceType": resource_type,
            "StorageConfig": storage_config,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe_all(hub, ctx, marker: str = None, page_size: int = None) -> Dict:
    r"""
    **Autogenerated function**

    Describes the current Elastic Load Balancing resource limits for your AWS account. For more information, see Limits for Your Classic Load Balancer in the Classic Load Balancers Guide.

    Args:
        marker(str, optional): The marker for the next set of results. (You received this marker from a previous call.). Defaults to None.
        page_size(int, optional): The maximum number of results to return with this call. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elb.account_limit.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elb.account_limit.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elb",
        operation="describe_account_limits",
        op_kwargs={"Marker": marker, "PageSize": page_size},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def copy(
    hub,
    ctx,
    source_snapshot_name: str,
    target_snapshot_name: str,
    target_bucket: str = None,
    kms_key_id: str = None,
    tags: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Makes a copy of an existing snapshot.  This operation is valid for Redis only.   Users or groups that have permissions to use the CopySnapshot operation can create their own Amazon S3 buckets and copy snapshots to it. To control access to your snapshots, use an IAM policy to control who has the ability to use the CopySnapshot operation. For more information about using IAM to control the use of ElastiCache operations, see Exporting Snapshots and Authentication & Access Control.  You could receive the following error messages.  Error Messages     Error Message: The S3 bucket %s is outside of the region.  Solution: Create an Amazon S3 bucket in the same region as your snapshot. For more information, see Step 1: Create an Amazon S3 Bucket in the ElastiCache User Guide.    Error Message: The S3 bucket %s does not exist.  Solution: Create an Amazon S3 bucket in the same region as your snapshot. For more information, see Step 1: Create an Amazon S3 Bucket in the ElastiCache User Guide.    Error Message: The S3 bucket %s is not owned by the authenticated user.  Solution: Create an Amazon S3 bucket in the same region as your snapshot. For more information, see Step 1: Create an Amazon S3 Bucket in the ElastiCache User Guide.    Error Message: The authenticated user does not have sufficient permissions to perform the desired activity.  Solution: Contact your system administrator to get the needed permissions.    Error Message: The S3 bucket %s already contains an object with key %s.  Solution: Give the TargetSnapshotName a new and unique value. If exporting a snapshot, you could alternatively create a new Amazon S3 bucket and use this same value for TargetSnapshotName.    Error Message:  ElastiCache has not been granted READ permissions %s on the S3 Bucket.  Solution: Add List and Read permissions on the bucket. For more information, see Step 2: Grant ElastiCache Access to Your Amazon S3 Bucket in the ElastiCache User Guide.    Error Message:  ElastiCache has not been granted WRITE permissions %s on the S3 Bucket.  Solution: Add Upload/Delete permissions on the bucket. For more information, see Step 2: Grant ElastiCache Access to Your Amazon S3 Bucket in the ElastiCache User Guide.    Error Message:  ElastiCache has not been granted READ_ACP permissions %s on the S3 Bucket.  Solution: Add View Permissions on the bucket. For more information, see Step 2: Grant ElastiCache Access to Your Amazon S3 Bucket in the ElastiCache User Guide.

    Args:
        source_snapshot_name(str): The name of an existing snapshot from which to make a copy.
        target_snapshot_name(str): A name for the snapshot copy. ElastiCache does not permit overwriting a snapshot, therefore this name must be unique within its context - ElastiCache or an Amazon S3 bucket if exporting.
        target_bucket(str, optional): The Amazon S3 bucket to which the snapshot is exported. This parameter is used only when exporting a snapshot for external access. When using this parameter to export a snapshot, be sure Amazon ElastiCache has the needed permissions to this S3 bucket. For more information, see Step 2: Grant ElastiCache Access to Your Amazon S3 Bucket in the Amazon ElastiCache User Guide. For more information, see Exporting a Snapshot in the Amazon ElastiCache User Guide. Defaults to None.
        kms_key_id(str, optional): The ID of the KMS key used to encrypt the target snapshot. Defaults to None.
        tags(List, optional): A list of tags to be added to this resource. A tag is a key-value pair. A tag key must be accompanied by a tag value, although null is accepted. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.snapshot.copy(
                ctx, source_snapshot_name=value, target_snapshot_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.snapshot.copy source_snapshot_name=value, target_snapshot_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="copy_snapshot",
        op_kwargs={
            "SourceSnapshotName": source_snapshot_name,
            "TargetSnapshotName": target_snapshot_name,
            "TargetBucket": target_bucket,
            "KmsKeyId": kms_key_id,
            "Tags": tags,
        },
    )


async def create(
    hub,
    ctx,
    snapshot_name: str,
    replication_group_id: str = None,
    cache_cluster_id: str = None,
    kms_key_id: str = None,
    tags: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a copy of an entire cluster or replication group at a specific moment in time.  This operation is valid for Redis only.

    Args:
        replication_group_id(str, optional): The identifier of an existing replication group. The snapshot is created from this replication group. Defaults to None.
        cache_cluster_id(str, optional): The identifier of an existing cluster. The snapshot is created from this cluster. Defaults to None.
        snapshot_name(str): A name for the snapshot being created.
        kms_key_id(str, optional): The ID of the KMS key used to encrypt the snapshot. Defaults to None.
        tags(List, optional): A list of tags to be added to this resource. A tag is a key-value pair. A tag key must be accompanied by a tag value, although null is accepted. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.snapshot.create(ctx, snapshot_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.snapshot.create snapshot_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="create_snapshot",
        op_kwargs={
            "ReplicationGroupId": replication_group_id,
            "CacheClusterId": cache_cluster_id,
            "SnapshotName": snapshot_name,
            "KmsKeyId": kms_key_id,
            "Tags": tags,
        },
    )


async def delete(hub, ctx, snapshot_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes an existing snapshot. When you receive a successful response from this operation, ElastiCache immediately begins deleting the snapshot; you cannot cancel or revert this operation.  This operation is valid for Redis only.

    Args:
        snapshot_name(str): The name of the snapshot to be deleted.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.snapshot.delete(ctx, snapshot_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.snapshot.delete snapshot_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="delete_snapshot",
        op_kwargs={"SnapshotName": snapshot_name},
    )


async def describe_all(
    hub,
    ctx,
    replication_group_id: str = None,
    cache_cluster_id: str = None,
    snapshot_name: str = None,
    snapshot_source: str = None,
    marker: str = None,
    max_records: int = None,
    show_node_group_config: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns information about cluster or replication group snapshots. By default, DescribeSnapshots lists all of your snapshots; it can optionally describe a single snapshot, or just the snapshots associated with a particular cache cluster.  This operation is valid for Redis only.

    Args:
        replication_group_id(str, optional): A user-supplied replication group identifier. If this parameter is specified, only snapshots associated with that specific replication group are described. Defaults to None.
        cache_cluster_id(str, optional): A user-supplied cluster identifier. If this parameter is specified, only snapshots associated with that specific cluster are described. Defaults to None.
        snapshot_name(str, optional): A user-supplied name of the snapshot. If this parameter is specified, only this snapshot are described. Defaults to None.
        snapshot_source(str, optional): If set to system, the output shows snapshots that were automatically created by ElastiCache. If set to user the output shows snapshots that were manually created. If omitted, the output shows both automatically and manually created snapshots. Defaults to None.
        marker(str, optional): An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by MaxRecords. Defaults to None.
        max_records(int, optional): The maximum number of records to include in the response. If more records exist than the specified MaxRecords value, a marker is included in the response so that the remaining results can be retrieved. Default: 50 Constraints: minimum 20; maximum 50. Defaults to None.
        show_node_group_config(bool, optional): A Boolean value which if true, the node group (shard) configuration is included in the snapshot description. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.snapshot.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.snapshot.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="describe_snapshots",
        op_kwargs={
            "ReplicationGroupId": replication_group_id,
            "CacheClusterId": cache_cluster_id,
            "SnapshotName": snapshot_name,
            "SnapshotSource": snapshot_source,
            "Marker": marker,
            "MaxRecords": max_records,
            "ShowNodeGroupConfig": show_node_group_config,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    name: str,
    definition: str,
    role_arn: str,
    type_: str = None,
    logging_configuration: Dict = None,
    tags: List = None,
    tracing_configuration: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a state machine. A state machine consists of a collection of states that can do work (Task states), determine to which states to transition next (Choice states), stop an execution with an error (Fail states), and so on. State machines are specified using a JSON-based, structured language. For more information, see Amazon States Language in the AWS Step Functions User Guide.  This operation is eventually consistent. The results are best effort and may not reflect very recent updates and changes.    CreateStateMachine is an idempotent API. Subsequent requests won’t create a duplicate resource if it was already created. CreateStateMachine's idempotency check is based on the state machine name, definition, type, LoggingConfiguration and TracingConfiguration. If a following request has a different roleArn or tags, Step Functions will ignore these differences and treat it as an idempotent request of the previous. In this case, roleArn and tags will not be updated, even if they are different.

    Args:
        name(str): The name of the state machine.  A name must not contain:   white space   brackets < > { } [ ]    wildcard characters ? *    special characters " # % \ ^ | ~ ` $ & , ; : /    control characters (U+0000-001F, U+007F-009F)   To enable logging with CloudWatch Logs, the name should only contain 0-9, A-Z, a-z, - and _.
        definition(str): The Amazon States Language definition of the state machine. See Amazon States Language.
        role_arn(str): The Amazon Resource Name (ARN) of the IAM role to use for this state machine.
        type_(str, optional): Determines whether a Standard or Express state machine is created. The default is STANDARD. You cannot update the type of a state machine once it has been created. Defaults to None.
        logging_configuration(Dict, optional): Defines what execution history events are logged and where they are logged.  By default, the level is set to OFF. For more information see Log Levels in the AWS Step Functions User Guide. . Defaults to None.
        tags(List, optional): Tags to be added when creating a state machine. An array of key-value pairs. For more information, see Using Cost Allocation Tags in the AWS Billing and Cost Management User Guide, and Controlling Access Using IAM Tags. Tags may only contain Unicode letters, digits, white space, or these symbols: _ . : / = + - @. Defaults to None.
        tracing_configuration(Dict, optional): Selects whether AWS X-Ray tracing is enabled. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.stepfunction.state_machine.init.create(
                ctx, name=value, definition=value, role_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.stepfunction.state_machine.init.create name=value, definition=value, role_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="stepfunctions",
        operation="create_state_machine",
        op_kwargs={
            "name": name,
            "definition": definition,
            "roleArn": role_arn,
            "type": type_,
            "loggingConfiguration": logging_configuration,
            "tags": tags,
            "tracingConfiguration": tracing_configuration,
        },
    )


async def delete(hub, ctx, state_machine_arn: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a state machine. This is an asynchronous operation: It sets the state machine's status to DELETING and begins the deletion process.   For EXPRESSstate machines, the deletion will happen eventually (usually less than a minute). Running executions may emit logs after DeleteStateMachine API is called.

    Args:
        state_machine_arn(str): The Amazon Resource Name (ARN) of the state machine to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.stepfunction.state_machine.init.delete(
                ctx, state_machine_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.stepfunction.state_machine.init.delete state_machine_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="stepfunctions",
        operation="delete_state_machine",
        op_kwargs={"stateMachineArn": state_machine_arn},
    )


async def describe(hub, ctx, state_machine_arn: str) -> Dict:
    r"""
    **Autogenerated function**

    Describes a state machine.  This operation is eventually consistent. The results are best effort and may not reflect very recent updates and changes.

    Args:
        state_machine_arn(str): The Amazon Resource Name (ARN) of the state machine to describe.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.stepfunction.state_machine.init.describe(
                ctx, state_machine_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.stepfunction.state_machine.init.describe state_machine_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="stepfunctions",
        operation="describe_state_machine",
        op_kwargs={"stateMachineArn": state_machine_arn},
    )


async def list_all(hub, ctx, max_results: int = None, next_token: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Lists the existing state machines. If nextToken is returned, there are more results available. The value of nextToken is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 24 hours. Using an expired pagination token will return an HTTP 400 InvalidToken error.  This operation is eventually consistent. The results are best effort and may not reflect very recent updates and changes.

    Args:
        max_results(int, optional): The maximum number of results that are returned per call. You can use nextToken to obtain further pages of results. The default is 100 and the maximum allowed page size is 1000. A value of 0 uses the default. This is only an upper limit. The actual number of results returned per call might be fewer than the specified maximum. Defaults to None.
        next_token(str, optional): If nextToken is returned, there are more results available. The value of nextToken is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 24 hours. Using an expired pagination token will return an HTTP 400 InvalidToken error. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.stepfunction.state_machine.init.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.stepfunction.state_machine.init.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="stepfunctions",
        operation="list_state_machines",
        op_kwargs={"maxResults": max_results, "nextToken": next_token},
    )


async def update(
    hub,
    ctx,
    state_machine_arn: str,
    definition: str = None,
    role_arn: str = None,
    logging_configuration: Dict = None,
    tracing_configuration: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Updates an existing state machine by modifying its definition, roleArn, or loggingConfiguration. Running executions will continue to use the previous definition and roleArn. You must include at least one of definition or roleArn or you will receive a MissingRequiredParameter error.  All StartExecution calls within a few seconds will use the updated definition and roleArn. Executions started immediately after calling UpdateStateMachine may use the previous state machine definition and roleArn.

    Args:
        state_machine_arn(str): The Amazon Resource Name (ARN) of the state machine.
        definition(str, optional): The Amazon States Language definition of the state machine. See Amazon States Language. Defaults to None.
        role_arn(str, optional): The Amazon Resource Name (ARN) of the IAM role of the state machine. Defaults to None.
        logging_configuration(Dict, optional): The LoggingConfiguration data type is used to set CloudWatch Logs options. Defaults to None.
        tracing_configuration(Dict, optional): Selects whether AWS X-Ray tracing is enabled. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.stepfunction.state_machine.init.update(
                ctx, state_machine_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.stepfunction.state_machine.init.update state_machine_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="stepfunctions",
        operation="update_state_machine",
        op_kwargs={
            "stateMachineArn": state_machine_arn,
            "definition": definition,
            "roleArn": role_arn,
            "loggingConfiguration": logging_configuration,
            "tracingConfiguration": tracing_configuration,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def translate(
    hub,
    ctx,
    text: str,
    source_language_code: str,
    target_language_code: str,
    terminology_names: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Translates input text from the source language to the target language. For a list of available languages and language codes, see what-is-languages.

    Args:
        text(str): The text to translate. The text string can be a maximum of 5,000 bytes long. Depending on your character set, this may be fewer than 5,000 characters.
        terminology_names(List, optional): The name of the terminology list file to be used in the TranslateText request. You can use 1 terminology list at most in a TranslateText request. Terminology lists can contain a maximum of 256 terms. Defaults to None.
        source_language_code(str): The language code for the language of the source text. The language must be a language supported by Amazon Translate. For a list of language codes, see what-is-languages. To have Amazon Translate determine the source language of your text, you can specify auto in the SourceLanguageCode field. If you specify auto, Amazon Translate will call Amazon Comprehend to determine the source language.
        target_language_code(str): The language code requested for the language of the target text. The language must be a language supported by Amazon Translate.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.translate.text.init.translate(
                ctx, text=value, source_language_code=value, target_language_code=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.translate.text.init.translate text=value, source_language_code=value, target_language_code=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="translate",
        operation="translate_text",
        op_kwargs={
            "Text": text,
            "TerminologyNames": terminology_names,
            "SourceLanguageCode": source_language_code,
            "TargetLanguageCode": target_language_code,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe_all(
    hub,
    ctx,
    service_code: str = None,
    format_version: str = None,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns the metadata for one service or a list of the metadata for all services. Use this without a service code to get the service codes for all services. Use it with a service code, such as AmazonEC2, to get information specific to that service, such as the attribute names available for that service. For example, some of the attribute names available for EC2 are volumeType, maxIopsVolume, operation, locationType, and instanceCapacity10xlarge.

    Args:
        service_code(str, optional): The code for the service whose information you want to retrieve, such as AmazonEC2. You can use the ServiceCode to filter the results in a GetProducts call. To retrieve a list of all services, leave this blank. Defaults to None.
        format_version(str, optional): The format version that you want the response to be in. Valid values are: aws_v1 . Defaults to None.
        next_token(str, optional): The pagination token that indicates the next set of results that you want to retrieve. Defaults to None.
        max_results(int, optional): The maximum number of results that you want returned in the response. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.pricing.service.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.pricing.service.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="pricing",
        operation="describe_services",
        op_kwargs={
            "ServiceCode": service_code,
            "FormatVersion": format_version,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def describe_all(
    hub,
    ctx,
    stack_name: str,
    stack_resource_drift_status_filters: List = None,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns drift information for the resources that have been checked for drift in the specified stack. This includes actual and expected configuration values for resources where CloudFormation detects configuration drift. For a given stack, there will be one StackResourceDrift for each stack resource that has been checked for drift. Resources that have not yet been checked for drift are not included. Resources that do not currently support drift detection are not checked, and so not included. For a list of resources that support drift detection, see Resources that Support Drift Detection. Use DetectStackResourceDrift to detect drift on individual resources, or DetectStackDrift to detect drift on all supported resources for a given stack.

    Args:
        stack_name(str): The name of the stack for which you want drift information.
        stack_resource_drift_status_filters(List, optional): The resource drift status values to use as filters for the resource drift results returned.    DELETED: The resource differs from its expected template configuration in that the resource has been deleted.    MODIFIED: One or more resource properties differ from their expected template values.    IN_SYNC: The resources's actual configuration matches its expected template configuration.    NOT_CHECKED: CloudFormation does not currently return this value.  . Defaults to None.
        next_token(str, optional): A string that identifies the next page of stack resource drift results. Defaults to None.
        max_results(int, optional): The maximum number of results to be returned with a single call. If the number of available results exceeds this maximum, the response includes a NextToken value that you can assign to the NextToken request parameter to get the next set of results. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudformation.stack.resource.drift.describe_all(
                ctx, stack_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudformation.stack.resource.drift.describe_all stack_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudformation",
        operation="describe_stack_resource_drifts",
        op_kwargs={
            "StackName": stack_name,
            "StackResourceDriftStatusFilters": stack_resource_drift_status_filters,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )


async def detect(hub, ctx, stack_name: str, logical_resource_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns information about whether a resource's actual configuration differs, or has drifted, from it's expected configuration, as defined in the stack template and any values specified as template parameters. This information includes actual and expected property values for resources in which CloudFormation detects drift. Only resource properties explicitly defined in the stack template are checked for drift. For more information about stack and resource drift, see Detecting Unregulated Configuration Changes to Stacks and Resources. Use DetectStackResourceDrift to detect drift on individual resources, or DetectStackDrift to detect drift on all resources in a given stack that support drift detection. Resources that do not currently support drift detection cannot be checked. For a list of resources that support drift detection, see Resources that Support Drift Detection.

    Args:
        stack_name(str): The name of the stack to which the resource belongs.
        logical_resource_id(str): The logical name of the resource for which to return drift information.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudformation.stack.resource.drift.detect(
                ctx, stack_name=value, logical_resource_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudformation.stack.resource.drift.detect stack_name=value, logical_resource_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudformation",
        operation="detect_stack_resource_drift",
        op_kwargs={"StackName": stack_name, "LogicalResourceId": logical_resource_id},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def join(
    hub,
    ctx,
    gateway_arn: str,
    domain_name: str,
    user_name: str,
    password: str,
    organizational_unit: str = None,
    domain_controllers: List = None,
    timeout_in_seconds: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Adds a file gateway to an Active Directory domain. This operation is only supported for file gateways that support the SMB file protocol.

    Args:
        gateway_arn(str): The Amazon Resource Name (ARN) of the gateway. Use the ListGateways operation to return a list of gateways for your account and Region.
        domain_name(str): The name of the domain that you want the gateway to join.
        organizational_unit(str, optional): The organizational unit (OU) is a container in an Active Directory that can hold users, groups, computers, and other OUs and this parameter specifies the OU that the gateway will join within the AD domain. Defaults to None.
        domain_controllers(List, optional): List of IPv4 addresses, NetBIOS names, or host names of your domain server. If you need to specify the port number include it after the colon (“:”). For example, mydc.mydomain.com:389. Defaults to None.
        timeout_in_seconds(int, optional): Specifies the time in seconds, in which the JoinDomain operation must complete. The default is 20 seconds. Defaults to None.
        user_name(str): Sets the user name of user who has permission to add the gateway to the Active Directory domain. The domain user account should be enabled to join computers to the domain. For example, you can use the domain administrator account or an account with delegated permissions to join computers to the domain.
        password(str): Sets the password of the user who has permission to add the gateway to the Active Directory domain.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.storagegateway.domain.join(
                ctx, gateway_arn=value, domain_name=value, user_name=value, password=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.storagegateway.domain.join gateway_arn=value, domain_name=value, user_name=value, password=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="storagegateway",
        operation="join_domain",
        op_kwargs={
            "GatewayARN": gateway_arn,
            "DomainName": domain_name,
            "OrganizationalUnit": organizational_unit,
            "DomainControllers": domain_controllers,
            "TimeoutInSeconds": timeout_in_seconds,
            "UserName": user_name,
            "Password": password,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def update(hub, ctx, assessment_id: str, status: str) -> Dict:
    r"""
    **Autogenerated function**

     Updates the status of an assessment in Audit Manager.

    Args:
        assessment_id(str):  The identifier for the specified assessment. .
        status(str):  The current status of the specified assessment. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.auditmanager.assessment.status.update(
                ctx, assessment_id=value, status=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.auditmanager.assessment.status.update assessment_id=value, status=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="auditmanager",
        operation="update_assessment_status",
        op_kwargs={"assessmentId": assessment_id, "status": status},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def get_all(hub, ctx, identities: List) -> Dict:
    r"""
    **Autogenerated function**

    Returns the current status of Easy DKIM signing for an entity. For domain name identities, this operation also returns the DKIM tokens that are required for Easy DKIM signing, and whether Amazon SES has successfully verified that these tokens have been published. This operation takes a list of identities as input and returns the following information for each:   Whether Easy DKIM signing is enabled or disabled.   A set of DKIM tokens that represent the identity. If the identity is an email address, the tokens represent the domain of that address.   Whether Amazon SES has successfully verified the DKIM tokens published in the domain's DNS. This information is only returned for domain name identities, not for email addresses.   This operation is throttled at one request per second and can only get DKIM attributes for up to 100 identities at a time. For more information about creating DNS records using DKIM tokens, go to the Amazon SES Developer Guide.

    Args:
        identities(List): A list of one or more verified identities - email addresses, domains, or both.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ses.identity.dkim.attribute.get_all(ctx, identities=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ses.identity.dkim.attribute.get_all identities=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ses",
        operation="get_identity_dkim_attributes",
        op_kwargs={"Identities": identities},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def start(hub, ctx, crawler_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Changes the schedule state of the specified crawler to SCHEDULED, unless the crawler is already running or the schedule state is already SCHEDULED.

    Args:
        crawler_name(str): Name of the crawler to schedule.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.glue.crawler.schedule.start(ctx, crawler_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.glue.crawler.schedule.start crawler_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="glue",
        operation="start_crawler_schedule",
        op_kwargs={"CrawlerName": crawler_name},
    )


async def stop(hub, ctx, crawler_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Sets the schedule state of the specified crawler to NOT_SCHEDULED, but does not stop the crawler if it is already running.

    Args:
        crawler_name(str): Name of the crawler whose schedule state to set.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.glue.crawler.schedule.stop(ctx, crawler_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.glue.crawler.schedule.stop crawler_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="glue",
        operation="stop_crawler_schedule",
        op_kwargs={"CrawlerName": crawler_name},
    )


async def update(hub, ctx, crawler_name: str, schedule: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Updates the schedule of a crawler using a cron expression.

    Args:
        crawler_name(str): The name of the crawler whose schedule to update.
        schedule(str, optional): The updated cron expression used to specify the schedule (see Time-Based Schedules for Jobs and Crawlers. For example, to run something every day at 12:15 UTC, you would specify: cron(15 12 * * ? *). Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.glue.crawler.schedule.update(ctx, crawler_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.glue.crawler.schedule.update crawler_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="glue",
        operation="update_crawler_schedule",
        op_kwargs={"CrawlerName": crawler_name, "Schedule": schedule},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def associate(
    hub,
    ctx,
    route_table_id: str,
    dry_run: bool = None,
    subnet_id: str = None,
    gateway_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Associates a subnet in your VPC or an internet gateway or virtual private gateway attached to your VPC with a route table in your VPC. This association causes traffic from the subnet or gateway to be routed according to the routes in the route table. The action returns an association ID, which you need in order to disassociate the route table later. A route table can be associated with multiple subnets. For more information, see Route tables in the Amazon Virtual Private Cloud User Guide.

    Args:
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        route_table_id(str): The ID of the route table.
        subnet_id(str, optional): The ID of the subnet. Defaults to None.
        gateway_id(str, optional): The ID of the internet gateway or virtual private gateway. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.route.table.init.associate(ctx, route_table_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.route.table.init.associate route_table_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="associate_route_table",
        op_kwargs={
            "DryRun": dry_run,
            "RouteTableId": route_table_id,
            "SubnetId": subnet_id,
            "GatewayId": gateway_id,
        },
    )


async def create(
    hub, ctx, vpc_id: str, dry_run: bool = None, tag_specifications: List = None
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a route table for the specified VPC. After you create a route table, you can add routes and associate the table with a subnet. For more information, see Route tables in the Amazon Virtual Private Cloud User Guide.

    Args:
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        vpc_id(str): The ID of the VPC.
        tag_specifications(List, optional): The tags to assign to the route table. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.route.table.init.create(ctx, vpc_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.route.table.init.create vpc_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="create_route_table",
        op_kwargs={
            "DryRun": dry_run,
            "VpcId": vpc_id,
            "TagSpecifications": tag_specifications,
        },
    )


async def delete(hub, ctx, route_table_id: str, dry_run: bool = None) -> None:
    r"""
    **Autogenerated function**

    Deletes the specified route table. You must disassociate the route table from any subnets before you can delete it. You can't delete the main route table.

    Args:
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        route_table_id(str): The ID of the route table.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.route.table.init.delete(ctx, route_table_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.route.table.init.delete route_table_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="delete_route_table",
        op_kwargs={"DryRun": dry_run, "RouteTableId": route_table_id},
    )


async def describe_all(
    hub,
    ctx,
    filters: List = None,
    dry_run: bool = None,
    route_table_ids: List = None,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Describes one or more of your route tables. Each subnet in your VPC must be associated with a route table. If a subnet is not explicitly associated with any route table, it is implicitly associated with the main route table. This command does not return the subnet ID for implicit associations. For more information, see Route tables in the Amazon Virtual Private Cloud User Guide.

    Args:
        filters(List, optional): One or more filters.    association.route-table-association-id - The ID of an association ID for the route table.    association.route-table-id - The ID of the route table involved in the association.    association.subnet-id - The ID of the subnet involved in the association.    association.main - Indicates whether the route table is the main route table for the VPC (true | false). Route tables that do not have an association ID are not returned in the response.    owner-id - The ID of the Amazon Web Services account that owns the route table.    route-table-id - The ID of the route table.    route.destination-cidr-block - The IPv4 CIDR range specified in a route in the table.    route.destination-ipv6-cidr-block - The IPv6 CIDR range specified in a route in the route table.    route.destination-prefix-list-id - The ID (prefix) of the Amazon Web Service specified in a route in the table.    route.egress-only-internet-gateway-id - The ID of an egress-only Internet gateway specified in a route in the route table.    route.gateway-id - The ID of a gateway specified in a route in the table.    route.instance-id - The ID of an instance specified in a route in the table.    route.nat-gateway-id - The ID of a NAT gateway.    route.transit-gateway-id - The ID of a transit gateway.    route.origin - Describes how the route was created. CreateRouteTable indicates that the route was automatically created when the route table was created; CreateRoute indicates that the route was manually added to the route table; EnableVgwRoutePropagation indicates that the route was propagated by route propagation.    route.state - The state of a route in the route table (active | blackhole). The blackhole state indicates that the route's target isn't available (for example, the specified gateway isn't attached to the VPC, the specified NAT instance has been terminated, and so on).    route.vpc-peering-connection-id - The ID of a VPC peering connection specified in a route in the table.    tag:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key Owner and the value TeamA, specify tag:Owner for the filter name and TeamA for the filter value.    tag-key - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.    vpc-id - The ID of the VPC for the route table.  . Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        route_table_ids(List, optional): One or more route table IDs. Default: Describes all your route tables. Defaults to None.
        next_token(str, optional): The token for the next page of results. Defaults to None.
        max_results(int, optional): The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned nextToken value. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.route.table.init.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.route.table.init.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="describe_route_tables",
        op_kwargs={
            "Filters": filters,
            "DryRun": dry_run,
            "RouteTableIds": route_table_ids,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )


async def disassociate(hub, ctx, association_id: str, dry_run: bool = None) -> None:
    r"""
    **Autogenerated function**

    Disassociates a subnet or gateway from a route table. After you perform this action, the subnet no longer uses the routes in the route table. Instead, it uses the routes in the VPC's main route table. For more information about route tables, see Route tables in the Amazon Virtual Private Cloud User Guide.

    Args:
        association_id(str): The association ID representing the current association between the route table and subnet or gateway.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.route.table.init.disassociate(ctx, association_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.route.table.init.disassociate association_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="disassociate_route_table",
        op_kwargs={"AssociationId": association_id, "DryRun": dry_run},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    name: str,
    vpc: str,
    creator_request_id: str = None,
    description: str = None,
    tags: List = None,
    properties: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a private namespace based on DNS, which is visible only inside a specified Amazon VPC. The namespace defines your service naming scheme. For example, if you name your namespace example.com and name your service backend, the resulting DNS name for the service is backend.example.com. Service instances that are registered using a private DNS namespace can be discovered using either a DiscoverInstances request or using DNS. For the current quota on the number of namespaces that you can create using the same account, see Cloud Map quotas in the Cloud Map Developer Guide.

    Args:
        name(str): The name that you want to assign to this namespace. When you create a private DNS namespace, Cloud Map automatically creates an Amazon Route 53 private hosted zone that has the same name as the namespace.
        creator_request_id(str, optional): A unique string that identifies the request and that allows failed CreatePrivateDnsNamespace requests to be retried without the risk of running the operation twice. CreatorRequestId can be any unique string (for example, a date/timestamp). Defaults to None.
        description(str, optional): A description for the namespace. Defaults to None.
        vpc(str): The ID of the Amazon VPC that you want to associate the namespace with.
        tags(List, optional): The tags to add to the namespace. Each tag consists of a key and an optional value that you define. Tags keys can be up to 128 characters in length, and tag values can be up to 256 characters in length. Defaults to None.
        properties(Dict, optional): Properties for the private DNS namespace. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.servicediscovery.private_dns_namespace.create(
                ctx, name=value, vpc=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.servicediscovery.private_dns_namespace.create name=value, vpc=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="servicediscovery",
        operation="create_private_dns_namespace",
        op_kwargs={
            "Name": name,
            "CreatorRequestId": creator_request_id,
            "Description": description,
            "Vpc": vpc,
            "Tags": tags,
            "Properties": properties,
        },
    )


async def update(
    hub, ctx, id_: str, namespace: Dict, updater_request_id: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Updates a private DNS namespace.

    Args:
        id_(str): The ID of the namespace that you want to update.
        updater_request_id(str, optional): A unique string that identifies the request and that allows failed UpdatePrivateDnsNamespace requests to be retried without the risk of running the operation twice. UpdaterRequestId can be any unique string (for example, a date/timestamp). Defaults to None.
        namespace(Dict): Updated properties for the private DNS namespace.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.servicediscovery.private_dns_namespace.update(
                ctx, id_=value, namespace=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.servicediscovery.private_dns_namespace.update id_=value, namespace=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="servicediscovery",
        operation="update_private_dns_namespace",
        op_kwargs={
            "Id": id_,
            "UpdaterRequestId": updater_request_id,
            "Namespace": namespace,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def cancel(hub, ctx, job_id: str, reason: str) -> Dict:
    r"""
    **Autogenerated function**

    Cancels a job in an Batch job queue. Jobs that are in the SUBMITTED, PENDING, or RUNNABLE state are canceled. Jobs that have progressed to STARTING or RUNNING aren't canceled, but the API operation still succeeds, even if no job is canceled. These jobs must be terminated with the TerminateJob operation.

    Args:
        job_id(str): The Batch job ID of the job to cancel.
        reason(str): A message to attach to the job that explains the reason for canceling it. This message is returned by future DescribeJobs operations on the job. This message is also recorded in the Batch activity logs.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.batch.job.init.cancel(ctx, job_id=value, reason=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.batch.job.init.cancel job_id=value, reason=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="batch",
        operation="cancel_job",
        op_kwargs={"jobId": job_id, "reason": reason},
    )


async def submit(
    hub,
    ctx,
    job_name: str,
    job_queue: str,
    job_definition: str,
    array_properties: Dict = None,
    depends_on: List = None,
    parameters: Dict = None,
    container_overrides: Dict = None,
    node_overrides: Dict = None,
    retry_strategy: Dict = None,
    propagate_tags: bool = None,
    timeout: Dict = None,
    tags: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Submits an Batch job from a job definition. Parameters that are specified during SubmitJob override parameters defined in the job definition. vCPU and memory requirements that are specified in the ResourceRequirements objects in the job definition are the exception. They can't be overridden this way using the memory and vcpus parameters. Rather, you must specify updates to job definition parameters in a ResourceRequirements object that's included in the containerOverrides parameter.  Jobs that run on Fargate resources can't be guaranteed to run for more than 14 days. This is because, after 14 days, Fargate resources might become unavailable and job might be terminated.

    Args:
        job_name(str): The name of the job. The first character must be alphanumeric, and up to 128 letters (uppercase and lowercase), numbers, hyphens, and underscores are allowed.
        job_queue(str): The job queue where the job is submitted. You can specify either the name or the Amazon Resource Name (ARN) of the queue.
        array_properties(Dict, optional): The array properties for the submitted job, such as the size of the array. The array size can be between 2 and 10,000. If you specify array properties for a job, it becomes an array job. For more information, see Array Jobs in the Batch User Guide. Defaults to None.
        depends_on(List, optional): A list of dependencies for the job. A job can depend upon a maximum of 20 jobs. You can specify a SEQUENTIAL type dependency without specifying a job ID for array jobs so that each child array job completes sequentially, starting at index 0. You can also specify an N_TO_N type dependency with a job ID for array jobs. In that case, each index child of this job must wait for the corresponding index child of each dependency to complete before it can begin. Defaults to None.
        job_definition(str): The job definition used by this job. This value can be one of name, name:revision, or the Amazon Resource Name (ARN) for the job definition. If name is specified without a revision then the latest active revision is used.
        parameters(Dict, optional): Additional parameters passed to the job that replace parameter substitution placeholders that are set in the job definition. Parameters are specified as a key and value pair mapping. Parameters in a SubmitJob request override any corresponding parameter defaults from the job definition. Defaults to None.
        container_overrides(Dict, optional): A list of container overrides in the JSON format that specify the name of a container in the specified job definition and the overrides it should receive. You can override the default command for a container, which is specified in the job definition or the Docker image, with a command override. You can also override existing environment variables on a container or add new environment variables to it with an environment override. Defaults to None.
        node_overrides(Dict, optional): A list of node overrides in JSON format that specify the node range to target and the container overrides for that node range.  This parameter isn't applicable to jobs that are running on Fargate resources; use containerOverrides instead. . Defaults to None.
        retry_strategy(Dict, optional): The retry strategy to use for failed jobs from this SubmitJob operation. When a retry strategy is specified here, it overrides the retry strategy defined in the job definition. Defaults to None.
        propagate_tags(bool, optional): Specifies whether to propagate the tags from the job or job definition to the corresponding Amazon ECS task. If no value is specified, the tags aren't propagated. Tags can only be propagated to the tasks during task creation. For tags with the same name, job tags are given priority over job definitions tags. If the total number of combined tags from the job and job definition is over 50, the job is moved to the FAILED state. When specified, this overrides the tag propagation setting in the job definition. Defaults to None.
        timeout(Dict, optional): The timeout configuration for this SubmitJob operation. You can specify a timeout duration after which Batch terminates your jobs if they haven't finished. If a job is terminated due to a timeout, it isn't retried. The minimum value for the timeout is 60 seconds. This configuration overrides any timeout configuration specified in the job definition. For array jobs, child jobs have the same timeout configuration as the parent job. For more information, see Job Timeouts in the Amazon Elastic Container Service Developer Guide. Defaults to None.
        tags(Dict, optional): The tags that you apply to the job request to help you categorize and organize your resources. Each tag consists of a key and an optional value. For more information, see Tagging Amazon Web Services Resources in Amazon Web Services General Reference. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.batch.job.init.submit(
                ctx, job_name=value, job_queue=value, job_definition=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.batch.job.init.submit job_name=value, job_queue=value, job_definition=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="batch",
        operation="submit_job",
        op_kwargs={
            "jobName": job_name,
            "jobQueue": job_queue,
            "arrayProperties": array_properties,
            "dependsOn": depends_on,
            "jobDefinition": job_definition,
            "parameters": parameters,
            "containerOverrides": container_overrides,
            "nodeOverrides": node_overrides,
            "retryStrategy": retry_strategy,
            "propagateTags": propagate_tags,
            "timeout": timeout,
            "tags": tags,
        },
    )


async def terminate(hub, ctx, job_id: str, reason: str) -> Dict:
    r"""
    **Autogenerated function**

    Terminates a job in a job queue. Jobs that are in the STARTING or RUNNING state are terminated, which causes them to transition to FAILED. Jobs that have not progressed to the STARTING state are cancelled.

    Args:
        job_id(str): The Batch job ID of the job to terminate.
        reason(str): A message to attach to the job that explains the reason for canceling it. This message is returned by future DescribeJobs operations on the job. This message is also recorded in the Batch activity logs.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.batch.job.init.terminate(ctx, job_id=value, reason=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.batch.job.init.terminate job_id=value, reason=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="batch",
        operation="terminate_job",
        op_kwargs={"jobId": job_id, "reason": reason},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create_multiple(
    hub,
    ctx,
    data_source_id: str,
    rds_data: Dict,
    role_arn: str,
    data_source_name: str = None,
    compute_statistics: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a DataSource object from an  Amazon Relational Database Service (Amazon RDS). A DataSource references data that can be used to perform CreateMLModel, CreateEvaluation, or CreateBatchPrediction operations.  CreateDataSourceFromRDS is an asynchronous operation. In response to CreateDataSourceFromRDS, Amazon Machine Learning (Amazon ML) immediately returns and sets the DataSource status to PENDING. After the DataSource is created and ready for use, Amazon ML sets the Status parameter to COMPLETED. DataSource in the COMPLETED or PENDING state can be used only to perform >CreateMLModel>, CreateEvaluation, or CreateBatchPrediction operations.   If Amazon ML cannot accept the input source, it sets the Status parameter to FAILED and includes an error message in the Message attribute of the GetDataSource operation response.

    Args:
        data_source_id(str): A user-supplied ID that uniquely identifies the DataSource. Typically, an Amazon Resource Number (ARN) becomes the ID for a DataSource.
        data_source_name(str, optional): A user-supplied name or description of the DataSource. Defaults to None.
        rds_data(Dict): The data specification of an Amazon RDS DataSource:   DatabaseInformation -    DatabaseName - The name of the Amazon RDS database.    InstanceIdentifier  - A unique identifier for the Amazon RDS database instance.     DatabaseCredentials - AWS Identity and Access Management (IAM) credentials that are used to connect to the Amazon RDS database.   ResourceRole - A role (DataPipelineDefaultResourceRole) assumed by an EC2 instance to carry out the copy task from Amazon RDS to Amazon Simple Storage Service (Amazon S3). For more information, see Role templates for data pipelines.   ServiceRole - A role (DataPipelineDefaultRole) assumed by the AWS Data Pipeline service to monitor the progress of the copy task from Amazon RDS to Amazon S3. For more information, see Role templates for data pipelines.   SecurityInfo - The security information to use to access an RDS DB instance. You need to set up appropriate ingress rules for the security entity IDs provided to allow access to the Amazon RDS instance. Specify a [SubnetId, SecurityGroupIds] pair for a VPC-based RDS DB instance.   SelectSqlQuery - A query that is used to retrieve the observation data for the Datasource.   S3StagingLocation - The Amazon S3 location for staging Amazon RDS data. The data retrieved from Amazon RDS using SelectSqlQuery is stored in this location.   DataSchemaUri - The Amazon S3 location of the DataSchema.   DataSchema - A JSON string representing the schema. This is not required if DataSchemaUri is specified.    DataRearrangement - A JSON string that represents the splitting and rearrangement requirements for the Datasource.   Sample -  "{\"splitting\":{\"percentBegin\":10,\"percentEnd\":60}}"   .
        role_arn(str): The role that Amazon ML assumes on behalf of the user to create and activate a data pipeline in the user's account and copy data using the SelectSqlQuery query from Amazon RDS to Amazon S3. .
        compute_statistics(bool, optional): The compute statistics for a DataSource. The statistics are generated from the observation data referenced by a DataSource. Amazon ML uses the statistics internally during MLModel training. This parameter must be set to true if the DataSource needs to be used for MLModel training. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.machinelearning.data_source.from_.rd.create_multiple(
                ctx, data_source_id=value, rds_data=value, role_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.machinelearning.data_source.from_.rd.create_multiple data_source_id=value, rds_data=value, role_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="machinelearning",
        operation="create_data_source_from_rds",
        op_kwargs={
            "DataSourceId": data_source_id,
            "DataSourceName": data_source_name,
            "RDSData": rds_data,
            "RoleARN": role_arn,
            "ComputeStatistics": compute_statistics,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    client_id: str,
    client_secret: str,
    grant_type: str,
    device_code: str,
    code: str = None,
    refresh_token: str = None,
    scope: List = None,
    redirect_uri: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates and returns an access token for the authorized client. The access token issued will be used to fetch short-term credentials for the assigned roles in the AWS account.

    Args:
        client_id(str): The unique identifier string for each client. This value should come from the persisted result of the RegisterClient API.
        client_secret(str): A secret string generated for the client. This value should come from the persisted result of the RegisterClient API.
        grant_type(str): Supports grant types for authorization code, refresh token, and device code request.
        device_code(str): Used only when calling this API for the device code grant type. This short-term code is used to identify this authentication attempt. This should come from an in-memory reference to the result of the StartDeviceAuthorization API.
        code(str, optional): The authorization code received from the authorization service. This parameter is required to perform an authorization grant request to get access to a token. Defaults to None.
        refresh_token(str, optional): The token used to obtain an access token in the event that the access token is invalid or expired. This token is not issued by the service. Defaults to None.
        scope(List, optional): The list of scopes that is defined by the client. Upon authorization, this list is used to restrict permissions when granting an access token. Defaults to None.
        redirect_uri(str, optional): The location of the application that will receive the authorization code. Users authorize the service to send the request to this location. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sso_oidc.token.create(
                ctx, client_id=value, client_secret=value, grant_type=value, device_code=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sso_oidc.token.create client_id=value, client_secret=value, grant_type=value, device_code=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sso-oidc",
        operation="create_token",
        op_kwargs={
            "clientId": client_id,
            "clientSecret": client_secret,
            "grantType": grant_type,
            "deviceCode": device_code,
            "code": code,
            "refreshToken": refresh_token,
            "scope": scope,
            "redirectUri": redirect_uri,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe(hub, ctx) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves information about the storage configuration for IoT SiteWise.


    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iotsitewise.storage_configuration.describe(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iotsitewise.storage_configuration.describe
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iotsitewise",
        operation="describe_storage_configuration",
        op_kwargs={},
    )


async def put(hub, ctx, storage_type: str, multi_layer_storage: Dict = None) -> Dict:
    r"""
    **Autogenerated function**

    Configures storage settings for IoT SiteWise.

    Args:
        storage_type(str): The type of storage that you specified for your data. The storage type can be one of the following values:    SITEWISE_DEFAULT_STORAGE – IoT SiteWise replicates your data into a service managed database.    MULTI_LAYER_STORAGE – IoT SiteWise replicates your data into a service managed database and saves a copy of your raw data and metadata in an Amazon S3 object that you specified.  .
        multi_layer_storage(Dict, optional): Identifies a storage destination. If you specified MULTI_LAYER_STORAGE for the storage type, you must specify a MultiLayerStorage object. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iotsitewise.storage_configuration.put(ctx, storage_type=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iotsitewise.storage_configuration.put storage_type=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iotsitewise",
        operation="put_storage_configuration",
        op_kwargs={
            "storageType": storage_type,
            "multiLayerStorage": multi_layer_storage,
        },
    )
